#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_switch_alternate_management_interface
short_description: Resource module for networks _switch _alternate _management _interface
description:
  - Manage operation update of the resource networks _switch _alternate _management
    _interface.
  - Update the switch alternate management interface for the network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  enabled:
    description: Boolean value to enable or disable AMI configuration. If enabled,
      VLAN and protocols must be set.
    type: bool
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  protocols:
    description: Can be one or more of the following values 'radius', 'snmp' or 'syslog'.
    elements: str
    type: list
  switches:
    description: Array of switch serial number and IP assignment. If parameter is
      present, it cannot have empty body. Note switches parameter is not applicable
      for template networks, in other words, do not put 'switches' in the body when
      updating template networks. Also, an empty 'switches' array will remove all
      previous assignments.
    elements: dict
    suboptions:
      alternateManagementIp:
        description: Switch alternative management IP. To remove a prior IP setting,
          provide an empty string.
        type: str
      gateway:
        description: Switch gateway must be in IP format. Only and must be specified
          for Polaris switches.
        type: str
      serial:
        description: Switch serial number.
        type: str
      subnetMask:
        description: Switch subnet mask must be in IP format. Only and must be specified
          for Polaris switches.
        type: str
    type: list
  vlanId:
    description: Alternate management VLAN, must be between 1 and 4094.
    type: int
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch updateNetworkSwitchAlternateManagementInterface
    description: Complete reference of the updateNetworkSwitchAlternateManagementInterface
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-alternate-management-interface
notes:
  - SDK Method used are
    switch.Switch.update_network_switch_alternate_management_interface,
  - Paths used are
    put /networks/{networkId}/switch/alternateManagementInterface,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_switch_alternate_management_interface:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    enabled: true
    networkId: string
    protocols:
      - radius
      - snmp
      - syslog
    switches:
      - alternateManagementIp: 1.2.3.4
        gateway: 1.2.3.5
        serial: Q234-ABCD-5678
        subnetMask: 255.255.255.0
    vlanId: 100
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "enabled": true,
      "useOobMgmt": true,
      "vlanId": 0,
      "protocols": [
        "string"
      ],
      "switches": [
        {
          "serial": "string",
          "alternateManagementIp": "string",
          "subnetMask": "string",
          "gateway": "string"
        }
      ]
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The eos bgp_address_family fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

import re

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.bgp_address_family.bgp_address_family import (
    Bgp_afArgs,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.rm_templates.bgp_address_family import (
    Bgp_afTemplate,
)


class Bgp_afFacts(object):
    """The eos bgp_address_family facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Bgp_afArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_config(self, connection):
        """Wrapper method for `connection.get()`
        This method exists solely to allow the unit test framework to mock device connection calls.
        """
        return connection.get("show running-config | section router\\sbgp ")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Bgp_af network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []

        if not data:
            data = self.get_config(connection)

        # remove global configs from bgp_address_family
        bgp_af_config = []
        vrf_set = ""
        start = False
        for bgp_line in data.splitlines():
            if "router bgp" in bgp_line:
                bgp_af_config.append(bgp_line)
            vrf_present = re.search(r"vrf\s\S+", bgp_line)
            if vrf_present:
                vrf_set = vrf_present.group(0)
            if start:
                bgp_af_config.append(bgp_line)
            if "address-family" in bgp_line:
                af_line = vrf_set + bgp_line
                bgp_af_config.append(af_line)
                start = True
            if start and "!" in bgp_line:
                start = False

        # parse native config using the Bgp_af template
        bgp_af_parser = Bgp_afTemplate(lines=bgp_af_config)
        objs = bgp_af_parser.parse()
        if objs:
            if "address_family" in objs:
                objs["address_family"] = list(objs["address_family"].values())
                for af in objs["address_family"]:
                    if "neighbor" in af:
                        af["neighbor"] = list(af["neighbor"].values())
                    if "network" in af:
                        af["network"] = list(af["network"].values())
                        af["network"] = sorted(
                            af["network"],
                            key=lambda k: k["address"],
                        )

        ansible_facts["ansible_network_resources"].pop(
            "bgp_address_family",
            None,
        )

        params = utils.remove_empties(
            utils.validate_config(self.argument_spec, {"config": objs}),
        )

        facts["bgp_address_family"] = params.get("config", [])
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
version_added: 3.2.0
module: cloudfront_response_headers_policy

short_description: Create, update and delete response headers policies to be used in a Cloudfront distribution

description:
  - Create, update and delete response headers policies to be used in a Cloudfront distribution for inserting custom headers
  - See docs at U(https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/cloudfront.html#CloudFront.Client.create_response_headers_policy)

author:
  - Stefan Horning (@stefanhorning)

options:
    state:
      description: Decides if the named policy should be absent or present
      choices:
        - present
        - absent
      default: present
      type: str
    name:
      description: Name of the policy
      required: true
      type: str
    comment:
      description: Description of the policy
      required: false
      type: str
    cors_config:
      description: CORS header config block
      required: false
      default: {}
      type: dict
    security_headers_config:
      description: Security headers config block. For headers suchs as XSS-Protection, Content-Security-Policy or Strict-Transport-Security
      required: false
      default: {}
      type: dict
    custom_headers_config:
      description: Custom headers config block. Define your own list of headers and values as a list
      required: false
      default: {}
      type: dict

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Creationg a Cloudfront header policy using all predefined header features and a custom header for demonstration
  community.aws.cloudfront_response_headers_policy:
    name: my-header-policy
    comment: My header policy for all the headers
    cors_config:
      access_control_allow_origins:
        items:
          - 'https://foo.com/bar'
          - 'https://bar.com/foo'
      access_control_allow_headers:
        items:
          - 'X-Session-Id'
      access_control_allow_methods:
        items:
          - GET
          - OPTIONS
          - HEAD
      access_control_allow_credentials: true
      access_control_expose_headers:
        items:
          - 'X-Session-Id'
      access_control_max_age_sec: 1800
      origin_override: true
    security_headers_config:
      xss_protection:
        protection: true
        report_uri: 'https://my.report-uri.com/foo/bar'
        override: true
      frame_options:
        frame_option: 'SAMEORIGIN'
        override: true
      referrer_policy:
        referrer_policy: 'same-origin'
        override: true
      content_security_policy:
        content_security_policy: "frame-ancestors 'none'; report-uri https://my.report-uri.com/r/d/csp/enforce;"
        override: true
      content_type_options:
        override: true
      strict_transport_security:
        include_subdomains: true
        preload: true
        access_control_max_age_sec: 63072000
        override: true
    custom_headers_config:
      items:
        - { header: 'X-Test-Header', value: 'Foo', override: true }
    state: present

- name: Delete header policy
  community.aws.cloudfront_response_headers_policy:
    name: my-header-policy
    state: absent
"""

RETURN = r"""
response_headers_policy:
    description: The policy's information
    returned: success
    type: complex
    contains:
      id:
        description: ID of the policy
        returned: always
        type: str
        sample: '10a45b52-630e-4b7c-77c6-205f06df0462'
      last_modified_time:
        description: Timestamp of last modification of policy
        returned: always
        type: str
        sample: '2022-02-04T13:23:27.304000+00:00'
      response_headers_policy_config:
        description: The response headers config dict containing all the headers configured
        returned: always
        type: complex
        contains:
          name:
            description: Name of the policy
            type: str
            returned: always
            sample: my-header-policy
"""

import datetime

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # caught by imported AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


class CloudfrontResponseHeadersPolicyService(object):
    def __init__(self, module):
        self.module = module
        self.client = module.client("cloudfront")
        self.check_mode = module.check_mode

    def find_response_headers_policy(self, name):
        try:
            policies = self.client.list_response_headers_policies()["ResponseHeadersPolicyList"]["Items"]
            for policy in policies:
                if policy["ResponseHeadersPolicy"]["ResponseHeadersPolicyConfig"]["Name"] == name:
                    # as the list_ request does not contain the Etag (which we need), we need to do another get_ request here
                    return self.client.get_response_headers_policy(Id=policy["ResponseHeadersPolicy"]["Id"])

        except (ClientError, BotoCoreError) as e:
            self.module.fail_json_aws(e, msg="Error fetching policy information")
        return None

    def create_response_header_policy(self, name, comment, cors_config, security_headers_config, custom_headers_config):
        cors_config = snake_dict_to_camel_dict(cors_config, capitalize_first=True)
        security_headers_config = snake_dict_to_camel_dict(security_headers_config, capitalize_first=True)

        # Little helper for turning xss_protection into XSSProtection and not into XssProtection
        if "XssProtection" in security_headers_config:
            security_headers_config["XSSProtection"] = security_headers_config.pop("XssProtection")

        custom_headers_config = snake_dict_to_camel_dict(custom_headers_config, capitalize_first=True)

        config = {
            "Name": name,
            "Comment": comment,
            "CorsConfig": self.insert_quantities(cors_config),
            "SecurityHeadersConfig": security_headers_config,
            "CustomHeadersConfig": self.insert_quantities(custom_headers_config),
        }

        config = {k: v for k, v in config.items() if v}

        matching_policy = self.find_response_headers_policy(name)

        changed = False

        if self.check_mode:
            self.module.exit_json(changed=True, response_headers_policy=camel_dict_to_snake_dict(config))

        if matching_policy is None:
            try:
                result = self.client.create_response_headers_policy(ResponseHeadersPolicyConfig=config)
                changed = True
            except (ClientError, BotoCoreError) as e:
                self.module.fail_json_aws(e, msg="Error creating policy")
        else:
            policy_id = matching_policy["ResponseHeadersPolicy"]["Id"]
            etag = matching_policy["ETag"]
            try:
                result = self.client.update_response_headers_policy(
                    Id=policy_id, IfMatch=etag, ResponseHeadersPolicyConfig=config
                )

                changed_time = result["ResponseHeadersPolicy"]["LastModifiedTime"]
                seconds = 3  # threshhold for returned timestamp age
                seconds_ago = datetime.datetime.now(changed_time.tzinfo) - datetime.timedelta(0, seconds)

                # consider change made by this execution of the module if returned timestamp was very recent
                if changed_time > seconds_ago:
                    changed = True
            except (ClientError, BotoCoreError) as e:
                self.module.fail_json_aws(e, msg="Updating creating policy")

        self.module.exit_json(changed=changed, **camel_dict_to_snake_dict(result))

    def delete_response_header_policy(self, name):
        matching_policy = self.find_response_headers_policy(name)

        if matching_policy is None:
            self.module.exit_json(msg="Didn't find a matching policy by that name, not deleting")
        else:
            policy_id = matching_policy["ResponseHeadersPolicy"]["Id"]
            etag = matching_policy["ETag"]
            if self.check_mode:
                result = {}
            else:
                try:
                    result = self.client.delete_response_headers_policy(Id=policy_id, IfMatch=etag)
                except (ClientError, BotoCoreError) as e:
                    self.module.fail_json_aws(e, msg="Error deleting policy")

            self.module.exit_json(changed=True, **camel_dict_to_snake_dict(result))

    # Inserts a Quantity field into dicts with a list ('Items')
    @staticmethod
    def insert_quantities(dict_with_items):
        # Items on top level case
        if "Items" in dict_with_items and isinstance(dict_with_items["Items"], list):
            dict_with_items["Quantity"] = len(dict_with_items["Items"])

        # Items on second level case
        for k, v in dict_with_items.items():
            if isinstance(v, dict) and "Items" in v:
                v["Quantity"] = len(v["Items"])

        return dict_with_items


def main():
    argument_spec = dict(
        name=dict(required=True, type="str"),
        comment=dict(type="str"),
        cors_config=dict(type="dict", default=dict()),
        security_headers_config=dict(type="dict", default=dict()),
        custom_headers_config=dict(type="dict", default=dict()),
        state=dict(choices=["present", "absent"], type="str", default="present"),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    name = module.params.get("name")
    comment = module.params.get("comment", "")
    cors_config = module.params.get("cors_config")
    security_headers_config = module.params.get("security_headers_config")
    custom_headers_config = module.params.get("custom_headers_config")
    state = module.params.get("state")

    service = CloudfrontResponseHeadersPolicyService(module)

    if state == "absent":
        service.delete_response_header_policy(name)
    else:
        service.create_response_header_policy(
            name, comment, cors_config, security_headers_config, custom_headers_config
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_device_connector_policy
short_description: Device Connector Policy configuration for Cisco Intersight
description:
  - Manages Device Connector Policy configuration on Cisco Intersight.
  - A policy to configure device connector settings for Cisco Intersight managed devices.
  - Enables or disables configuration lockout on the endpoint to prevent local configuration changes.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Device Connector Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Device Connector Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
    default: []
  enable_lockout:
    description:
      - Enables configuration lockout on the endpoint.
      - When enabled, prevents local configuration changes on the device through IMC or CIMC interfaces.
      - Lockout ensures that all configuration changes must be made through Intersight.
    type: bool
    default: false
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create Device Connector Policy with lockout enabled
  cisco.intersight.intersight_device_connector_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "device-connector-lockout-policy"
    description: "Device connector policy with lockout enabled"
    enable_lockout: true
    tags:
      - Key: "Environment"
        Value: "Production"
    state: present

- name: Create Device Connector Policy with lockout disabled
  cisco.intersight.intersight_device_connector_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "device-connector-no-lockout-policy"
    description: "Device connector policy with lockout disabled"
    enable_lockout: false
    state: present

- name: Update existing Device Connector Policy
  cisco.intersight.intersight_device_connector_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "device-connector-lockout-policy"
    description: "Updated device connector policy"
    enable_lockout: false
    state: present

- name: Delete Device Connector Policy
  cisco.intersight.intersight_device_connector_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "device-connector-lockout-policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "device-connector-lockout-policy",
        "ObjectType": "deviceconnector.Policy",
        "Description": "Device connector policy with lockout enabled",
        "LockoutEnabled": true,
        "Organization": {
            "Moid": "675450ee69726530014753e2",
            "ObjectType": "organization.Organization"
        },
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict', default=[]),
        enable_lockout=dict(type='bool', default=False)
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    intersight.api_body = {
        'Organization': {
            'Name': module.params['organization'],
        },
        'Name': module.params['name']
    }
    if module.params['state'] == 'present':
        intersight.set_tags_and_description()
        intersight.api_body['LockoutEnabled'] = module.params['enable_lockout']

    resource_path = '/deviceconnector/Policies'
    intersight.configure_policy_or_profile(resource_path=resource_path)
    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Service parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def handleTimestamp(config_data):
    command = "service timestamps"
    command += " " + config_data.get("msg") if config_data.get("msg") else ""
    command += " " + config_data.get("timestamp") if config_data.get("timestamp") else ""

    if config_data.get("datetime_options"):
        datetime_op = config_data.get("datetime_options")
        command += " msec" if datetime_op.get("msec") else ""
        command += " localtime" if datetime_op.get("localtime") else ""
        command += " show-timezone" if datetime_op.get("show_timezone") else ""
        command += " year" if datetime_op.get("year") else ""

    return command


class ServiceTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(ServiceTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "call_home",
            "getval": re.compile(
                r"""
                ^service\scall-home
                """, re.VERBOSE,
            ),
            "setval": "service call-home",
            "result": {
                "call_home": True,
            },
        },
        {
            "name": "compress_config",
            "getval": re.compile(
                r"""
                ^service\scompress-config
                """, re.VERBOSE,
            ),
            "setval": "service compress-config",
            "result": {
                "compress_config": True,
            },
        },
        {
            "name": "config",
            "getval": re.compile(
                r"""
                ^service\sconfig
                """, re.VERBOSE,
            ),
            "setval": "service config",
            "result": {
                "config": True,
            },
        },
        {
            "name": "counters",
            "getval": re.compile(
                r"""
                ^service\scounters\smax\sage(\s(?P<max_age>\d+))?
                """, re.VERBOSE,
            ),
            "setval": "service counters max age {{ counters }}",
            "result": {
                "counters": "{{ max_age|int }}",
            },
        },
        {
            "name": "dhcp",
            "getval": re.compile(
                r"""
                ^service\sdhcp
                """, re.VERBOSE,
            ),
            "setval": "service dhcp",
            "result": {
                "dhcp": True,
            },
        },
        {
            "name": "disable_ip_fast_frag",
            "getval": re.compile(
                r"""
                ^service\sdisable-ip-fast-frag
                """, re.VERBOSE,
            ),
            "setval": "service disable-ip-fast-frag",
            "result": {
                "disable_ip_fast_frag": True,
            },
        },
        {
            "name": "exec_callback",
            "getval": re.compile(
                r"""
                ^service\sexec-callback
                """, re.VERBOSE,
            ),
            "setval": "service exec-callback",
            "result": {
                "exec_callback": True,
            },
        },
        {
            "name": "exec_wait",
            "getval": re.compile(
                r"""
                ^service\sexec-wait
                """, re.VERBOSE,
            ),
            "setval": "service exec-wait",
            "return": {
                "exec_wait": True,
            },
        },
        {
            "name": "hide_telnet_addresses",
            "getval": re.compile(
                r"""
                ^service\shide-telnet-addresses
                """, re.VERBOSE,
            ),
            "setval": "service hide-telnet-addresses",
            "result": {
                "hide_telnet_addresses": True,
            },
        },
        {
            "name": "internal",
            "getval": re.compile(
                r"""
                ^service\sinternal
                """, re.VERBOSE,
            ),
            "setval": "service internal",
            "result": {
                "internal": True,
            },
        },
        {
            "name": "linenumber",
            "getval": re.compile(
                r"""
                ^service\slinenumber
                """, re.VERBOSE,
            ),
            "setval": "service linenumber",
            "result": {
                "linenumber": True,
            },
        },
        {
            "name": "log",
            "getval": re.compile(
                r"""
                ^service\slog\sbacktrace?
                """, re.VERBOSE,
            ),
            "setval": "service log backtrace",
            "result": {
                "log": True,
            },
        },
        {
            "name": "log_hidden",
            "getval": re.compile(
                r"""
                ^service\slog-hidden
                """, re.VERBOSE,
            ),
            "setval": "service log-hidden",
            "result": {
                "log_hidden": True,
            },
        },
        {
            "name": "nagle",
            "getval": re.compile(
                r"""
                ^service\snagle
                """, re.VERBOSE,
            ),
            "setval": "service nagle",
            "result": {
                "nagle": True,
            },
        },
        {
            "name": "old_slip_prompts",
            "getval": re.compile(
                r"""
                ^service\sold-slip-prompts
                """, re.VERBOSE,
            ),
            "setval": "service old-slip-prompts",
            "result": {
                "old_slip_prompts": True,
            },
        },
        {
            "name": "pad",
            "getval": re.compile(
                r"""
                ^service\spad$
                """, re.VERBOSE,
            ),
            "setval": "service pad",
            "result": {
                "pad": True,
            },
        },
        {
            "name": "pad_cmns",
            "getval": re.compile(
                r"""
                ^service\spad\scmns
                """, re.VERBOSE,
            ),
            "setval": "service pad cmns",
            "result": {
                "pad_cmns": True,
            },
        },
        {
            "name": "pad_from_xot",
            "getval": re.compile(
                r"""
                ^service\spad\sfrom-xot
                """, re.VERBOSE,
            ),
            "setval": "service pad from-xot",
            "result": {
                "pad_from_xot": True,
            },
        },
        {
            "name": "pad_to_xot",
            "getval": re.compile(
                r"""
                ^service\spad\sto-xot
                """, re.VERBOSE,
            ),
            "setval": "service pad to-xot",
            "result": {
                "pad_to_xot": True,
            },
        },
        {
            "name": "password_encryption",
            "getval": re.compile(
                r"""
                ^service\spassword-encryption
                """, re.VERBOSE,
            ),
            "setval": "service password-encryption",
            "result": {
                "password_encryption": True,
            },
        },
        {
            "name": "password_recovery",
            "getval": re.compile(
                r"""
                ^service\spassword-recovery
                """, re.VERBOSE,
            ),
            "setval": "service password-recovery",
            "remval": "service password-recovery\nyes",
            "result": {
                "password_recovery": True,
            },
        },
        {
            "name": "prompt",
            "getval": re.compile(
                r"""
                ^service\sprompt\sconfig
                """, re.VERBOSE,
            ),
            "setval": "service prompt config",
            "result": {
                "prompt": True,
            },
        },
        {
            "name": "private_config_encryption",
            "getval": re.compile(
                r"""
                ^service\sprivate-config-encryption
                """, re.VERBOSE,
            ),
            "setval": "service private-config-encryption",
            "result": {
                "private_config_encryption": True,
            },
        },
        {
            "name": "pt_vty_logging",
            "getval": re.compile(
                r"""
                ^service\spt-vty-logging
                """, re.VERBOSE,
            ),
            "setval": "service pt-vty-logging",
            "result": {
                "pt_vty_logging": True,
            },
        },
        {
            "name": "scripting",
            "getval": re.compile(
                r"""
                ^service\sscripting
                """, re.VERBOSE,
            ),
            "setval": "service scripting",
            "result": {
                "scripting": True,
            },
        },
        {
            "name": "sequence_numbers",
            "getval": re.compile(
                r"""
                ^service\ssequence-numbers
                """, re.VERBOSE,
            ),
            "setval": "service sequence-numbers",
            "result": {
                "sequence_numbers": True,
            },
        },
        {
            "name": "slave_coredump",
            "getval": re.compile(
                r"""
                ^service\sslave-coredump
                """, re.VERBOSE,
            ),
            "setval": "service slave-coredump",
            "result": {
                "slave_coredump": True,
            },
        },
        {
            "name": "slave_log",
            "getval": re.compile(
                r"""
                ^service\sslave-log
                """, re.VERBOSE,
            ),
            "setval": "service slave-log",
            "result": {
                "slave_log": True,
            },
        },
        {
            "name": "tcp_keepalives_in",
            "getval": re.compile(
                r"""
                ^service\stcp-keepalives-in
                """, re.VERBOSE,
            ),
            "setval": "service tcp-keepalives-in",
            "result": {
                "tcp_keepalives_in": True,
            },
        },
        {
            "name": "tcp_keepalives_out",
            "getval": re.compile(
                r"""
                ^service\stcp-keepalives-out
                """, re.VERBOSE,
            ),
            "setval": "service tcp-keepalives-out",
            "result": {
                "tcp_keepalives_out": True,
            },
        },
        {
            "name": "tcp_small_servers",
            "getval": re.compile(
                r"""
                ^service\stcp-small-servers
                (\s(?P<max_servers>\d+))?
                (\s(?P<no_limit>no-limit))?
                """, re.VERBOSE,
            ),
            "setval": "service tcp-small-servers"
            "{{ (' ' + tcp_small_servers.max_servers|string) if tcp_small_servers.max_servers is defined else '' }}"
            "{{ (' no-limit') if tcp_small_servers.no_limit|d(False) else '' }}",
            "result": {
                "tcp_small_servers": {
                    "enable": True,
                    "max_servers": "{{ max_servers }}",
                    "no_limit": "{{ not not no_limit }}",
                },
            },
        },
        {
            "name": "udp_small_servers",
            "getval": re.compile(
                r"""
                ^service\sudp-small-servers
                (\s(?P<max_servers>\d+))?
                (\s(?P<no_limit>no-limit))?
                """, re.VERBOSE,
            ),
            "setval": "{{ ('service udp-small-servers') if udp_small_servers.enable|d(False) else '' }}"
            "{{ (' ' + udp_small_servers.max_servers|string) if udp_small_servers.max_servers is defined else '' }}"
            "{{ (' no-limit') if udp_small_servers.no_limit|d(False) else '' }}",
            "result": {
                "udp_small_servers": {
                    "enable": True,
                    "max_servers": "{{ max_servers }}",
                    "no_limit": "{{ not not no_limit }}",
                },
            },
        },
        {
            "name": "telnet_zeroidle",
            "getval": re.compile(
                r"""
                ^service\stelnet-zeroidle
                """, re.VERBOSE,
            ),
            "setval": "service telnet-zeroidle",
            "result": {
                "telnet_zeroidle": True,
            },
        },
        {
            "name": "log_timestamps",
            "getval": re.compile(
                r"""
                ^service\stimestamps\slog
                (\s(?P<timestamp>\S+))?
                (\s(?P<msec>msec))?
                (\s(?P<localtime>localtime))?
                (\s(?P<show_timezone>show-timezone))?
                (\s(?P<year>year))?
                """, re.VERBOSE,
            ),
            "remval": "service timestamps log",
            "setval": handleTimestamp,
            "result": {
                "timestamps": [
                    {
                        "msg": "log",
                        "timestamp": "{{ timestamp if timestamp is defined else 'uptime' }}",
                        "datetime_options": {
                            "msec": "{{ True if msec else False}}",
                            "localtime": "{{ True if localtime else False }}",
                            "show_timezone": "{{ True if show_timezone else False }}",
                            "year": "{{ True if year else False }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "debug_timestamps",
            "getval": re.compile(
                r"""
                ^service\stimestamps\sdebug
                (\s(?P<timestamp>\S+))?
                (\s(?P<msec>msec))?
                (\s(?P<localtime>localtime))?
                (\s(?P<show_timezone>show-timezone))?
                (\s(?P<year>year))?
                """, re.VERBOSE,
            ),
            "remval": "service timestamps debug",
            "setval": handleTimestamp,
            "result": {
                "timestamps": [
                    {
                        "msg": "debug",
                        "timestamp": "{{ timestamp if timestamp is defined else 'uptime' }}",
                        "datetime_options": {
                            "msec": "{{ True if msec else False}}",
                            "localtime": "{{ True if localtime else False }}",
                            "show_timezone": "{{ True if show_timezone else False }}",
                            "year": "{{ True if year else False }}",
                        },
                    },
                ],
            },
        },
        {
            "name": "unsupported_transceiver",
            "getval": re.compile(
                r"""
                ^service\sunsupported-transceiver
                """, re.VERBOSE,
            ),
            "setval": "service unsupported-transceiver",
            "result": {
                "unsupported_transceiver": True,
            },
        },
    ]



# Copyright (c) 2022, Julien Riou <julien@riou.xyz>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

from ansible.errors import AnsibleFilterError
from ansible.module_utils.common.collections import is_string
from ansible.module_utils.common.text.converters import to_bytes

try:
    from zlib import crc32

    HAS_ZLIB = True
except ImportError:
    HAS_ZLIB = False


DOCUMENTATION = r"""
name: crc32
short_description: Generate a CRC32 checksum
version_added: 5.4.0
description:
  - Checksum a string using CRC32 algorithm and return its hexadecimal representation.
options:
  _input:
    description:
      - The string to checksum.
    type: string
    required: true
author:
  - Julien Riou
"""

EXAMPLES = r"""
- name: Checksum a test string
  ansible.builtin.debug:
    msg: "{{ 'test' | community.general.crc32 }}"
"""

RETURN = r"""
_value:
  description: CRC32 checksum.
  type: string
"""


def crc32s(value):
    if not is_string(value):
        raise AnsibleFilterError(f"Invalid value type ({type(value)}) for crc32 ({value!r})")

    if not HAS_ZLIB:
        raise AnsibleFilterError("Failed to import zlib module")

    data = to_bytes(value, errors="surrogate_or_strict")
    return f"{crc32(data) & 0xFFFFFFFF:x}"


class FilterModule:
    def filters(self):
        return {
            "crc32": crc32s,
        }



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_appliance_vpn_third_party_vpn_peers
short_description: Resource module for organizations _appliance _vpn _third _party
  _vpn _peers
description:
  - Manage operation update of the resource organizations _appliance _vpn _third _party
    _vpn _peers.
  - Update the third party VPN peers for an organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  peers:
    description: The list of VPN peers.
    elements: dict
    suboptions:
      ebgpNeighbor:
        description: Optional The BGP neighbor configuration for the VPN peer. Supported
          only for MX 19.1 and above.
        suboptions:
          ebgpHoldTimer:
            description: The eBGP hold timer in seconds for each neighbor. The eBGP
              hold timer must be an integer between 12 and 240.
            type: int
          ebgpMultihop:
            description: Configure this if the neighbor is not adjacent. The eBGP
              multi-hop must be an integer between 1 and 255.
            type: int
          ipVersion:
            description: The IP version of the neighbor.
            type: int
          multiExitDiscriminator:
            description: Configures the local metric associated with routes received
              from the remote peer. Routes from peers with lower metrics are will
              be preferred. Must be an integer between 0 and 4294967295. MED is 6th
              in the decision tree when identical routes from multiple peers exist.
            type: int
          neighborIp:
            description: IPv4/IPv6 address of the neighbor.
            type: str
          pathPrepend:
            description: Prepends the AS_PATH BGP Attribute associated with routes
              received from the remote peer. Configurable value of ASNs to prepend.
              Length of the array may not exceed 10, and each ASN in the array must
              be an integer between 1 and 4294967295. AS_PATH is 4th in the decision
              tree when identical routes from multiple peers exist.
            elements: int
            type: list
          remoteAsNumber:
            description: Remote ASN of the neighbor. The remote ASN must be an integer
              between 1 and 4294967295.
            type: int
          sourceIp:
            description: Source IP of eBGP neighbor.
            type: str
          weight:
            description: Configures the local metric associated with routes received
              from the remote peer. Routes from peers with lower metrics are will
              be preferred. Must be an integer between 0 and 4294967295. MED is 6th
              in the decision tree when identical routes from multiple peers exist.
            type: int
        type: dict
      ecmpUplinkConfigs:
        description: Optional The ECMP per-uplink BGP-over-IPsec configuration for
          the VPN peer.
        elements: dict
        suboptions:
          ebgpNeighbor:
            description: Optional The BGP neighbor configuration for the VPN peer.
              Supported only for MX 19.1 and above.
            suboptions:
              neighborIp:
                description: IPv4/IPv6 address of the neighbor.
                type: str
              sourceIp:
                description: Source IP of eBGP neighbor.
                type: str
            type: dict
          id:
            description: ID of the ECMP uplink configuration.
            type: str
          privateSubnets:
            description: The list of the private subnets of the VPN peer.
            elements: str
            type: list
          wan:
            description: The WAN uplink associated with this ECMP configuration.
            type: str
        type: list
      group:
        description: Optional Contains the mapping between primary tunnel and backup
          tunnels.
        suboptions:
          activeActiveTunnel:
            description: Optional Both primary and backup tunnels are active.
            type: bool
          failover:
            description: Optional Contains the failover configuration for the group.
            suboptions:
              directToInternet:
                description: Optional When both primary and backup tunnels are down,
                  direct traffic to the internet. Traffic will be routed via the WAN.
                type: bool
            type: dict
          number:
            description: Optional Represents the ordering of primary and backup tunnels
              group. Primary and backup tunnels are grouped by this number. If you
              submit a request with the numbers 1, 9, 999, these numbers will be automatically
              adjusted to a sequential order starting from 1. So, they will be changed
              to 1, 2, 3 to reflect their positions in the sequence.
            type: int
        type: dict
      ikeVersion:
        description: Optional The IKE version to be used for the IPsec VPN peer configuration.
          Defaults to '1' when omitted.
        type: str
      ipsecPolicies:
        description: Custom IPSec policies for the VPN peer. If not included and a
          preset has not been chosen, the default preset for IPSec policies will be
          used.
        suboptions:
          childAuthAlgo:
            description: This is the authentication algorithms to be used in Phase
              2. The value should be an array with one of the following algorithms
              'sha256', 'sha1', 'md5'.
            elements: str
            type: list
          childCipherAlgo:
            description: This is the cipher algorithms to be used in Phase 2. The
              value should be an array with one or more of the following algorithms
              'aes256', 'aes192', 'aes128', 'tripledes', 'des', 'null'.
            elements: str
            type: list
          childLifetime:
            description: The lifetime of the Phase 2 SA in seconds.
            type: int
          childPfsGroup:
            description: This is the Diffie-Hellman group to be used for Perfect Forward
              Secrecy in Phase 2. The value should be an array with one of the following
              values 'disabled','group14', 'group5', 'group2', 'group1'.
            elements: str
            type: list
          ikeAuthAlgo:
            description: This is the authentication algorithm to be used in Phase
              1. The value should be an array with one of the following algorithms
              'sha256', 'sha1', 'md5'.
            elements: str
            type: list
          ikeCipherAlgo:
            description: This is the cipher algorithm to be used in Phase 1. The value
              should be an array with one of the following algorithms 'aes256', 'aes192',
              'aes128', 'tripledes', 'des'.
            elements: str
            type: list
          ikeDiffieHellmanGroup:
            description: This is the Diffie-Hellman group to be used in Phase 1. The
              value should be an array with one of the following algorithms 'group14',
              'group5', 'group2', 'group1'.
            elements: str
            type: list
          ikeLifetime:
            description: The lifetime of the Phase 1 SA in seconds.
            type: int
          ikePrfAlgo:
            description: Optional This is the pseudo-random function to be used in
              IKE_SA. The value should be an array with one of the following algorithms
              'prfsha256', 'prfsha1', 'prfmd5', 'default'. The 'default' option can
              be used to default to the Authentication algorithm.
            elements: str
            type: list
        type: dict
      ipsecPoliciesPreset:
        description: One of the following available presets 'default', 'aws', 'azure',
          'umbrella', 'zscaler'. If this is provided, the 'ipsecPolicies' parameter
          is ignored.
        type: str
      isRouteBased:
        description: Optional If true, the VPN peer is route-based. If not included,
          the default is false. Supported only for MX 19.1 and above.
        type: bool
      localId:
        description: Optional The local ID is used to identify the MX to the peer.
          This will apply to all MXs this peer applies to.
        type: str
      name:
        description: The name of the VPN peer.
        type: str
      network:
        description: Optional A list of network Names and IDs that will connect with
          this peer. Supported only for MX 19.1 and above.
        suboptions:
          ids:
            description: Optional A list of network IDs.
            elements: str
            type: list
        type: dict
      networkTags:
        description: A list of network tags that will connect with this peer. Use
          'all' for all networks. Use 'none' for no networks. If not included, the
          default is 'all'.
        elements: str
        type: list
      peerId:
        description: The ID of the IPsec peer.
        type: str
      priorityInGroup:
        description: Optional Represents the order of peer inside a group. If you
          submit a request with the numbers 1, 9, 999, these numbers will be automatically
          adjusted to a sequential order starting from 1. So, they will be changed
          to 1, 2, 3 to reflect their positions in the sequence.
        type: int
      privateSubnets:
        description: The list of the private subnets of the VPN peer.
        elements: str
        type: list
      publicHostname:
        description: Optional The public hostname of the VPN peer.
        type: str
      publicIp:
        description: Optional The public IP of the VPN peer.
        type: str
      remoteId:
        description: Optional The remote ID is used to identify the connecting VPN
          peer. This can either be a valid IPv4 Address, FQDN or User FQDN.
        type: str
      secret:
        description: The shared secret with the VPN peer.
        type: str
      slaPolicy:
        description: Optional Information about the SLA policy to be applied to the
          peer.
        suboptions:
          id:
            description: The ID of the SLA policy.
            type: str
        type: dict
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance updateOrganizationApplianceVpnThirdPartyVPNPeers
    description: Complete reference of the updateOrganizationApplianceVpnThirdPartyVPNPeers
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-appliance-vpn-third-party-vpn-peers
notes:
  - SDK Method used are
    appliance.Appliance.update_organization_appliance_vpn_third_party_vpnpeers,
  - Paths used are
    put /organizations/{organizationId}/appliance/vpn/thirdPartyVPNPeers,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.organizations_appliance_vpn_third_party_vpnpeers:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    organizationId: string
    peers:
      - ebgpNeighbor:
          ebgpHoldTimer: 180
          ebgpMultihop: 2
          ipVersion: 4
          multiExitDiscriminator: 1
          neighborIp: 10.10.10.22
          pathPrepend:
            - 1
            - 2
          remoteAsNumber: 64343
          sourceIp: 10.10.10.22
          weight: 10
        ecmpUplinkConfigs:
          - ebgpNeighbor:
              neighborIp: 169.254.10.2
              sourceIp: 169.254.10.1
            id: '7890'
            privateSubnets:
              - 169.254.10.0/30
            wan: WAN 1
        group:
          activeActiveTunnel: true
          failover:
            directToInternet: true
          number: 1
        ikeVersion: '2'
        ipsecPolicies:
          childAuthAlgo:
            - sha1
          childCipherAlgo:
            - aes128
          childLifetime: 28800
          childPfsGroup:
            - disabled
          ikeAuthAlgo:
            - sha1
          ikeCipherAlgo:
            - tripledes
          ikeDiffieHellmanGroup:
            - group2
          ikeLifetime: 28800
          ikePrfAlgo:
            - prfsha1
        ipsecPoliciesPreset: default
        isRouteBased: true
        localId: myMXId@meraki.com
        name: Peer Name
        network:
          ids:
            - N_1
            - L_2
            - N_3
        networkTags:
          - none
        peerId: '1234'
        priorityInGroup: 1
        privateSubnets:
          - 192.168.1.0/24
          - 192.168.128.0/24
        publicHostname: example.com
        publicIp: 123.123.123.1
        remoteId: miles@meraki.com
        secret: Sample Password
        slaPolicy:
          id: '1234'
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    [
      {
        "peerId": "string",
        "name": "string",
        "publicIp": "string",
        "remoteId": "string",
        "localId": "string",
        "secret": "string",
        "privateSubnets": [
          "string"
        ],
        "ipsecPolicies": {
          "ikeCipherAlgo": [
            "string"
          ],
          "ikeAuthAlgo": [
            "string"
          ],
          "ikePrfAlgo": [
            "string"
          ],
          "ikeDiffieHellmanGroup": [
            "string"
          ],
          "ikeLifetime": 0,
          "childCipherAlgo": [
            "string"
          ],
          "childAuthAlgo": [
            "string"
          ],
          "childPfsGroup": [
            "string"
          ],
          "childLifetime": 0
        },
        "slaPolicy": {
          "id": "string"
        },
        "ipsecPoliciesPreset": "string",
        "ikeVersion": "string",
        "networkTags": [
          "string"
        ],
        "network": {
          "names": [
            "string"
          ],
          "ids": [
            "string"
          ]
        },
        "isRouteBased": true,
        "ebgpNeighbor": {
          "neighborId": 0,
          "neighborIp": "string",
          "ipVersion": 0,
          "remoteAsNumber": 0,
          "ebgpHoldTimer": 0,
          "ebgpMultihop": 0,
          "sourceIp": "string",
          "pathPrepend": [
            0
          ],
          "multiExitDiscriminator": 0,
          "weight": 0
        },
        "ecmpUplinkConfigs": [
          {
            "id": "string",
            "wan": "string",
            "privateSubnets": [
              "string"
            ],
            "ebgpNeighbor": {
              "neighborIp": "string",
              "sourceIp": "string"
            }
          }
        ],
        "priorityInGroup": 0,
        "group": {
          "number": 0,
          "failover": {
            "directToInternet": true
          },
          "activeActiveTunnel": true
        }
      }
    ]
"""



# Copyright (c) Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:

    # Standard documentation fragment
    DOCUMENTATION = r"""
options: {}
attributes:
  check_mode:
    description: Can run in C(check_mode) and return changed status prediction without modifying target.
  diff_mode:
    description: Will return details on what has changed (or possibly needs changing in C(check_mode)), when in diff mode.
  idempotent:
    description:
      - When run twice in a row outside check mode, with the same arguments, the second invocation indicates no change.
      - This assumes that the system controlled/queried by the module has not changed in a relevant way.
"""

    # Should be used together with the standard fragment
    IDEMPOTENT_NOT_MODIFY_STATE = r"""
options: {}
attributes:
  idempotent:
    support: full
    details:
      - This action does not modify state.
"""

    # Should be used together with the standard fragment
    INFO_MODULE = r"""
options: {}
attributes:
  check_mode:
    support: full
    details:
      - This action does not modify state.
  diff_mode:
    support: N/A
    details:
      - This action does not modify state.
"""

    ACTIONGROUP_HETZNER = r"""
options: {}
attributes:
  action_group:
    description: Use C(group/community.dns.hetzner) in C(module_defaults) to set defaults for this module.
    support: full
    membership:
      - community.dns.hetzner
"""

    ACTIONGROUP_HOSTTECH = r"""
options: {}
attributes:
  action_group:
    description: Use C(group/community.dns.hosttech) in C(module_defaults) to set defaults for this module.
    support: full
    membership:
      - community.dns.hosttech
"""

    CONN = r"""
options: {}
attributes:
  become:
    description: Is usable alongside C(become) keywords.
  connection:
    description: Uses the target's configured connection information to execute code on it.
  delegation:
    description: Can be used in conjunction with C(delegate_to) and related keywords.
"""

    FACTS = r"""
options: {}
attributes:
  facts:
    description: Action returns an C(ansible_facts) dictionary that will update existing host facts.
"""

    # Should be used together with the standard fragment and the FACTS fragment
    FACTS_MODULE = r"""
options: {}
attributes:
  check_mode:
    support: full
    details:
      - This action does not modify state.
  diff_mode:
    support: N/A
    details:
      - This action does not modify state.
  facts:
    support: full
"""

    FILES = r"""
options: {}
attributes:
  safe_file_operations:
    description: Uses Ansible's strict file operation functions to ensure proper permissions and avoid data corruption.
"""

    FLOW = r"""
options: {}
attributes:
  action:
    description: Indicates this has a corresponding action plugin so some parts of the options can be executed on the controller.
  async:
    description: Supports being used with the C(async) keyword.
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2022, Tim Cragg (@timcragg)
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_fabric_pod_policy_group
short_description: Manage Fabric Pod Policy Groups (fabric:PodPGrp)
description:
- Fabric Pod Policy Group (fabric:PodPGrp) configuration on Cisco ACI fabrics.
options:
  name:
    description:
    - Name of the policy group
    type: str
    aliases: [ policy_group, policy_group_name, pod_policy_group ]
  date_time_policy:
    description:
    - NTP policy to bind to the policy group
    type: str
    aliases: [ ntp_policy ]
  isis_policy:
    description:
    - IS-IS policy to bind to the policy group
    type: str
  coop_group_policy:
    description:
    - COOP group policy to bind to the policy group
    type: str
    aliases: [ coop_policy ]
  bgp_rr_policy:
    description:
    - BGP route reflector policy to bind to the policy group
    type: str
  management_access_policy:
    description:
    - Management access policy to bind to the policy group
    type: str
    aliases: [ management_policy, mgmt_policy ]
  snmp_policy:
    description:
    - SNMP policy to bind to the policy group
    type: str
  macsec_policy:
    description:
    - MACSec policy to bind to the policy group
    type: str
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(fabricPodPGrp).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
"""

EXAMPLES = r"""
- name: Add a new fabric pod policy group
  cisco.aci.aci_fabric_pod_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: my_pod_pol_grp
    snmp_policy: my_snmp_pol
    bgp_rr_policy: my_bgp_rr_pol
    state: present
  delegate_to: localhost

- name: Remove a fabric pod policy group
  cisco.aci.aci_fabric_pod_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: my_pod_pol_grp
    state: absent
  delegate_to: localhost

- name: Query a fabric pod policy group
  cisco.aci.aci_fabric_pod_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: my_pod_pol_grp
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all fabric pod policy groups
  cisco.aci.aci_fabric_pod_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
   current:
     description: The existing configuration from the APIC after the module has finished
     returned: success
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production environment",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   error:
     description: The error information as returned from the APIC
     returned: failure
     type: dict
     sample:
       {
           "code": "122",
           "text": "unknown managed object class foo"
       }
   raw:
     description: The raw output returned by the APIC REST API (xml or json)
     returned: parse error
     type: str
     sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class "/></imdata>'
   sent:
     description: The actual/minimal configuration pushed to the APIC
     returned: info
     type: list
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment"
               }
           }
       }
   previous:
     description: The original configuration from the APIC before the module has started
     returned: info
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   proposed:
     description: The assembled configuration from the user-provided parameters
     returned: info
     type: dict
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment",
                   "name": "production"
               }
           }
       }
   filter_string:
     description: The filter string used for the request
     returned: failure or debug
     type: str
     sample: ?rsp-prop-include=config-only
   method:
     description: The HTTP method used for the request to the APIC
     returned: failure or debug
     type: str
     sample: POST
   response:
     description: The HTTP response from the APIC
     returned: failure or debug
     type: str
     sample: OK (30 bytes)
   status:
     description: The HTTP status from the APIC
     returned: failure or debug
     type: int
     sample: 200
   url:
     description: The HTTP url used for the request to the APIC
     returned: failure or debug
     type: str
     sample: https://10.11.12.13/api/mo/uni/tn-production.json
   """

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        name=dict(type="str", aliases=["policy_group", "policy_group_name", "pod_policy_group"]),
        date_time_policy=dict(type="str", aliases=["ntp_policy"]),
        isis_policy=dict(type="str"),
        coop_group_policy=dict(type="str", aliases=["coop_policy"]),
        bgp_rr_policy=dict(type="str"),
        management_access_policy=dict(type="str", aliases=["management_policy", "mgmt_policy"]),
        snmp_policy=dict(type="str"),
        macsec_policy=dict(type="str"),
        name_alias=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name"]],
            ["state", "present", ["name"]],
        ],
    )
    aci = ACIModule(module)

    name = module.params.get("name")
    date_time_policy = module.params.get("date_time_policy")
    isis_policy = module.params.get("isis_policy")
    coop_group_policy = module.params.get("coop_group_policy")
    bgp_rr_policy = module.params.get("bgp_rr_policy")
    management_access_policy = module.params.get("management_access_policy")
    snmp_policy = module.params.get("snmp_policy")
    macsec_policy = module.params.get("macsec_policy")
    name_alias = module.params.get("name_alias")
    state = module.params.get("state")
    child_classes = [
        "fabricRsSnmpPol",
        "fabricRsPodPGrpIsisDomP",
        "fabricRsPodPGrpCoopP",
        "fabricRsPodPGrpBGPRRP",
        "fabricRsTimePol",
        "fabricRsMacsecPol",
        "fabricRsCommPol",
    ]

    aci.construct_url(
        root_class=dict(
            aci_class="fabricPodPGrp",
            aci_rn="fabric/funcprof/podpgrp-{0}".format(name),
            module_object=name,
            target_filter={"name": name},
        ),
        child_classes=child_classes,
    )

    aci.get_existing()

    if state == "present":
        child_configs = []
        if date_time_policy is not None:
            child_configs.append(dict(fabricRsTimePol=dict(attributes=dict(tnDatetimePolName=date_time_policy))))
        if isis_policy is not None:
            child_configs.append(dict(fabricRsPodPGrpIsisDomP=dict(attributes=dict(tnIsisDomPolName=isis_policy))))
        if coop_group_policy is not None:
            child_configs.append(dict(fabricRsPodPGrpCoopP=dict(attributes=dict(tnCoopPolName=coop_group_policy))))
        if bgp_rr_policy is not None:
            child_configs.append(dict(fabricRsPodPGrpBGPRRP=dict(attributes=dict(tnBgpInstPolName=bgp_rr_policy))))
        if management_access_policy is not None:
            child_configs.append(dict(fabricRsCommPol=dict(attributes=dict(tnCommPolName=management_access_policy))))
        if snmp_policy is not None:
            child_configs.append(dict(fabricRsSnmpPol=dict(attributes=dict(tnSnmpPolName=snmp_policy))))
        if macsec_policy is not None:
            child_configs.append(dict(fabricRsMacsecPol=dict(attributes=dict(tnMacsecFabIfPolName=macsec_policy))))
        aci.payload(
            aci_class="fabricPodPGrp",
            class_config=dict(name=name, nameAlias=name_alias),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="fabricPodPGrp")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
# Python SDK Reference: https://learn.microsoft.com/en-us/python/api/azure-mgmt-cdn/azure.mgmt.cdn.operations.rulesoperations?view=azure-python
#

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_afdrules_info
version_added: "3.4.0"
short_description: Get Azure Front Door Rule facts to be used with Standard or Premium Frontdoor Service
description:
    - Get facts for a specific Azure Front Door (AFD) Rule or all AFD Rules in a Ruleset.
    - This differs from the Front Door classic service and only is intended to be used by the Standard or Premium service offering.

options:
    name:
        description:
            - Name of the delivery rule which is unique within the endpoint.
        type: str
    profile_name:
        description:
            - Name of the Azure Front Door Standard or Azure Front Door Premium profile which is unique within the resource group
        required: true
        type: str
    resource_group:
        description:
            - Name of the Resource group within the Azure subscription.
        required: true
        type: str
    rule_set_name:
        description:
            - Name of the rule set under the profile.
        required: true
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Jarret Tooley (@jartoo)
'''

EXAMPLES = '''
- name: Get facts for all Rules in the AFD Profile
  azure_rm_afdrule_info:
    rule_set_name: myRuleset
    profile_name: myProfile
    resource_group: myResourceGroup

- name: Get facts of specific AFD Rule
  azure_rm_afdrule_info:
    name: myRule1
    rule_set_name: myRuleset
    profile_name: myProfile
    resource_group: myResourceGroup
'''

RETURN = '''
afdrules:
    description: List of AFD Rules.
    returned: always
    type: complex
    contains:
        actions:
            description:
                - A list of actions that are executed when all the conditions of a rule are satisfied.
            type: list
            elements: dict
            contains:
                cache_behavior:
                    description:
                        - Caching behavior for the requests.
                    type: str
                cache_duration:
                    description:
                        - The duration for which the content needs to be cached. Allowed format is [d.]hh:mm:ss.
                    type: str
                custom_fragment:
                    description:
                        - Fragment to add to the redirect URL. Fragment is the part of the URL that comes after #. Do not include the #
                    type: str
                custom_hostname:
                    description:
                        - Host to redirect. Leave empty to use the incoming host as the destination host.
                    type: str
                custom_path:
                    description:
                        - The full path to redirect. Path cannot be empty and must start with /. Leave empty to use the incoming path as destination path.
                    type: str
                custom_query_string:
                    description:
                        - The set of query strings to be placed in the redirect URL.
                        - Setting this value would replace any existing query string; leave empty to preserve the incoming query string.
                        - Query string must be in <key>=:code:<value> format. ? and & will be added automatically so do not include them.
                    type: str
                destination:
                    description:
                        - Define the relative URL to which the above requests will be rewritten by.
                    type: str
                destination_protocol:
                    description:
                        - Protocol to use for the redirect.
                    type: str
                forwarding_protocol:
                    description:
                        - Protocol this rule will use when forwarding traffic to backends.
                    type: str
                header_action:
                    description:
                        - Action to perform.
                    type: str
                header_name:
                    description:
                        - Name of the header to modify.
                    type: str
                is_compression_enabled:
                    description:
                        - The caching configuration for this route. Indicates whether content compression is enabled on AzureFrontDoor.
                        - If compression is enabled, content will be served as compressed if user requests for a compressed version.
                        - Content won't be compressed on AzureFrontDoor when requested content is smaller than 1 byte or larger than 1 MB.
                    type: str
                name:
                    description:
                        - The name of the action for the delivery rule.
                    type: str
                origin_group:
                    description:
                        - defines the OriginGroup that would override the DefaultOriginGroup.
                    type: str
                preserve_unmatched_path:
                    description:
                        - Whether to preserve unmatched path.
                    type: bool
                query_parameters:
                    description:
                        - query parameters to include or exclude (comma separated).
                    type: str
                query_string_caching_behavior:
                    description:
                        - The caching configuration for this route. Defines how Frontdoor caches requests that include query strings.
                        - You can ignore any query strings when caching, ignore specific query strings, cache every request with a unique URL,
                        - or cache specific query strings.
                    type: str
                redirect_type:
                    description:
                        - The redirect type the rule will use when redirecting traffic.
                    type: str
                source_pattern:
                    description:
                        - Define a request URI pattern that identifies the type of requests that may be rewritten.
                        - If value is blank, all strings are matched.
                    type: str
                value:
                    description:
                        - Value for the specified action.
                    type: str
        conditions:
            description:
                - A list of conditions that must be matched for the actions to be executed.
            type: list
            elements: dict
            contains:
                name:
                    description:
                        - The name of the condition for the delivery rule.
                    type: str
                operator:
                    description:
                        - Describes operator to be matched.
                    type: str
                match_values:
                    description:
                        - The match value for the condition of the delivery rule.
                    type: list
                    elements: str
                negate_condition:
                    description:
                        - Describes if this is a negate condition or not.
                    type: bool
                selector:
                    description:
                        - Name of item to be matched.
                    type: str
                transforms:
                    description:
                        - List of transforms.
                    type: list
                    elements: str
        match_processing_behavior:
            description:
                - If this rule is a match should the rules engine continue running the remaining rules or stop.
            type: str
        name:
            description:
                - Name of the delivery rule which is unique within the endpoint.
            type: str
        order:
            description:
                - The order in which the rules are applied for the endpoint.
                - A rule with a lesser order will be applied before a rule with a greater order. Rule with order 0 is a special rule.
                - It does not require any condition and actions listed in it will always be applied.
            type: int
        profile_name:
            description:
                - Name of the Azure Front Door Standard or Azure Front Door Premium profile which is unique within the resource group.
            type: str
        resource_group:
            description:
                - Name of the Resource group within the Azure subscription.
            type: str
        rule_set_name:
            description:
                - Name of the rule set under the profile.
            type: str
        state:
            description:
                - Assert the state of the CDN profile. Use C(present) to create or update a CDN profile and C(absent) to delete it.
            type: str
        type:
            description:
                - Resource type.
            type: str
'''

# import re
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


AZURE_OBJECT_CLASS = 'AFDRules'


class AzureRMAFDRuleInfo(AzureRMModuleBase):
    """Utility class to get Azure AFD Rule facts"""

    def __init__(self):

        self.module_args = dict(
            name=dict(type='str'),
            profile_name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True),
            rule_set_name=dict(type='str', required=True)
        )

        self.results = dict(
            changed=False,
            afdrules=[]
        )

        self.name = None
        self.profile_name = None
        self.resource_group = None
        self.rule_set_name = None

        super(AzureRMAFDRuleInfo, self).__init__(
            supports_check_mode=True,
            derived_arg_spec=self.module_args,
            supports_tags=False,
            facts_module=True
        )

    def exec_module(self, **kwargs):

        for key in self.module_args:
            setattr(self, key, kwargs[key])

        if self.name:
            self.results['afdrules'] = self.get_item()
        else:
            self.results['afdrules'] = self.list_by_endpoint()

        return self.results

    def get_item(self):
        """Get a single Azure AFD Rule"""

        self.log('Get properties for {0}'.format(self.name))

        item = None
        result = []

        try:
            item = self.cdn_client.rules.get(resource_group_name=self.resource_group,
                                             profile_name=self.profile_name,
                                             rule_set_name=self.rule_set_name,
                                             rule_name=self.name)
        except Exception as exc:
            self.log("Failed to find an existing resource. {0}".format(str(exc)))

        if item:
            result = [self.serialize_afdrule(item)]

        return result

    def list_by_endpoint(self):
        """Get all Azure AFD Rules within an AFD profile"""

        self.log('List all AFD Rules within an AFD profile')

        try:
            response = self.cdn_client.rules.list_by_rule_set(resource_group_name=self.resource_group,
                                                              profile_name=self.profile_name,
                                                              rule_set_name=self.rule_set_name)
        except Exception as exc:
            self.fail('Failed to list all items - {0}'.format(str(exc)))

        results = []
        for item in response:
            results.append(self.serialize_afdrule(item))

        return results

    def serialize_afdrule(self, rules):
        '''
        Convert a AFD Rule object to dict.
        :param afdrule: AFD Rule object
        :return: dict
        '''
        return dict(
            actions=parse_action_condition(rules.actions),
            conditions=parse_action_condition(rules.conditions),
            deployment_status=rules.deployment_status,
            id=rules.id,
            match_processing_behavior=rules.match_processing_behavior,
            name=rules.name,
            order=rules.order,
            provisioning_state=rules.provisioning_state,
            rule_set_name=rules.rule_set_name,
            type=rules.type
        )


def parse_action_condition(items):
    ''' Convert Actions and Conditions objects to list '''

    parsed = []
    for item in items:
        new_item = {}
        vitem = vars(item)
        for field in vitem:
            if field == 'parameters':
                subvitem = vars(vitem.get('parameters'))
                for subfield in subvitem:
                    if subfield == 'origin_group_override':
                        ogo = subvitem["origin_group_override"]
                        new_item["forwarding_protocol"] = None
                        new_item['origin_group_id'] = None
                        if ogo:
                            new_item["forwarding_protocol"] = ogo.forwarding_protocol
                            og = ogo.origin_group
                            if og:
                                new_item['origin_group_id'] = og.id
                    elif subfield == 'cache_configuration':
                        cc = subvitem["cache_configuration"]
                        new_item["query_string_caching_behavior"] = None
                        new_item['query_parameters'] = None
                        new_item['is_compression_enabled'] = None
                        new_item['cache_behavior'] = None
                        new_item['cache_duration'] = None
                        if cc:
                            new_item["query_string_caching_behavior"] = ogo.query_string_caching_behavior
                            new_item["query_parameters"] = ogo.query_parameters
                            new_item["is_compression_enabled"] = ogo.is_compression_enabled
                            new_item["cache_behavior"] = ogo.cache_behavior
                            new_item["cache_duration"] = ogo.cache_duration
                    elif subfield != 'additional_properties':
                        new_item[subfield] = subvitem[subfield]
            elif field != 'additional_properties':
                new_item[field] = vitem[field]
        parsed.append(new_item)
    return parsed


def main():
    """Main module execution code path"""
    AzureRMAFDRuleInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2019 Dell EMC Inc.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: idrac_redfish_config
short_description: Manages servers through iDRAC using Dell Redfish APIs
description:
  - For use with Dell iDRAC operations that require Redfish OEM extensions.
  - Builds Redfish URIs locally and sends them to remote iDRAC controllers to set or update a configuration attribute.
extends_documentation_fragment:
  - community.general._attributes
  - community.general._redfish
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  category:
    required: true
    type: str
    description:
      - Category to execute on iDRAC.
  command:
    required: true
    description:
      - List of commands to execute on iDRAC.
      - V(SetManagerAttributes), V(SetLifecycleControllerAttributes) and V(SetSystemAttributes) are mutually exclusive commands
        when O(category) is V(Manager).
    type: list
    elements: str
  baseuri:
    required: true
    description:
      - Base URI of iDRAC.
    type: str
  username:
    description:
      - Username for authenticating to iDRAC.
    type: str
  password:
    description:
      - Password for authenticating to iDRAC.
    type: str
  auth_token:
    description:
      - Security token for authenticating to iDRAC.
    type: str
    version_added: 2.3.0
  manager_attributes:
    description:
      - Dictionary of iDRAC attribute name and value pairs to update.
    default: {}
    type: 'dict'
    version_added: '0.2.0'
  timeout:
    description:
      - Timeout in seconds for HTTP requests to iDRAC.
    default: 10
    type: int
  resource_id:
    description:
      - ID of the System, Manager or Chassis to modify.
    type: str
    version_added: '0.2.0'
  validate_certs:
    version_added: 10.6.0
  ca_path:
    version_added: 10.6.0
  ciphers:
    version_added: 10.6.0

author: "Jose Delarosa (@jose-delarosa)"
"""

EXAMPLES = r"""
- name: Enable NTP and set NTP server and Time zone attributes in iDRAC
  community.general.idrac_redfish_config:
    category: Manager
    command: SetManagerAttributes
    resource_id: iDRAC.Embedded.1
    manager_attributes:
      NTPConfigGroup.1.NTPEnable: "Enabled"
      NTPConfigGroup.1.NTP1: "{{ ntpserver1 }}"
      Time.1.Timezone: "{{ timezone }}"
    baseuri: "{{ baseuri }}"
    username: "{{ username}}"
    password: "{{ password }}"

- name: Enable Syslog and set Syslog servers in iDRAC
  community.general.idrac_redfish_config:
    category: Manager
    command: SetManagerAttributes
    resource_id: iDRAC.Embedded.1
    manager_attributes:
      SysLog.1.SysLogEnable: "Enabled"
      SysLog.1.Server1: "{{ syslog_server1 }}"
      SysLog.1.Server2: "{{ syslog_server2 }}"
    baseuri: "{{ baseuri }}"
    username: "{{ username}}"
    password: "{{ password }}"

- name: Configure SNMP community string, port, protocol and trap format
  community.general.idrac_redfish_config:
    category: Manager
    command: SetManagerAttributes
    resource_id: iDRAC.Embedded.1
    manager_attributes:
      SNMP.1.AgentEnable: "Enabled"
      SNMP.1.AgentCommunity: "public_community_string"
      SNMP.1.TrapFormat: "SNMPv1"
      SNMP.1.SNMPProtocol: "All"
      SNMP.1.DiscoveryPort: 161
      SNMP.1.AlertPort: 162
    baseuri: "{{ baseuri }}"
    username: "{{ username}}"
    password: "{{ password }}"

- name: Enable CSIOR
  community.general.idrac_redfish_config:
    category: Manager
    command: SetLifecycleControllerAttributes
    resource_id: iDRAC.Embedded.1
    manager_attributes:
      LCAttributes.1.CollectSystemInventoryOnRestart: "Enabled"
    baseuri: "{{ baseuri }}"
    username: "{{ username}}"
    password: "{{ password }}"

- name: Set Power Supply Redundancy Policy to A/B Grid Redundant
  community.general.idrac_redfish_config:
    category: Manager
    command: SetSystemAttributes
    resource_id: iDRAC.Embedded.1
    manager_attributes:
      ServerPwr.1.PSRedPolicy: "A/B Grid Redundant"
    baseuri: "{{ baseuri }}"
    username: "{{ username}}"
    password: "{{ password }}"
"""

RETURN = r"""
msg:
  description: Message with action result or error description.
  returned: always
  type: str
  sample: "Action was successful"
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.validation import check_mutually_exclusive, check_required_arguments

from ansible_collections.community.general.plugins.module_utils._redfish_utils import (
    REDFISH_COMMON_ARGUMENT_SPEC,
    RedfishUtils,
)


class IdracRedfishUtils(RedfishUtils):
    def set_manager_attributes(self, command):
        result = {}
        required_arg_spec = {"manager_attributes": {"required": True}}

        try:
            check_required_arguments(required_arg_spec, self.module.params)

        except TypeError as e:
            self.module.fail_json(msg=f"{e}")

        key = "Attributes"
        command_manager_attributes_uri_map = {
            "SetManagerAttributes": self.manager_uri,
            "SetLifecycleControllerAttributes": "/redfish/v1/Managers/LifecycleController.Embedded.1",
            "SetSystemAttributes": "/redfish/v1/Managers/System.Embedded.1",
        }
        manager_uri = command_manager_attributes_uri_map.get(command, self.manager_uri)

        attributes = self.module.params["manager_attributes"]

        attrs_to_patch = {}
        attrs_skipped = {}
        attrs_bad = {}  # Store attrs which were not found in the system

        # Search for key entry and extract URI from it
        response = self.get_request(f"{self.root_uri}{manager_uri}/{key}")
        if response["ret"] is False:
            return response
        result["ret"] = True
        data = response["data"]

        if key not in data:
            return {"ret": False, "msg": f"{command}: Key {key} not found", "warning": ""}

        for attr_name, attr_value in attributes.items():
            # Check if attribute exists
            if attr_name not in data["Attributes"]:
                # Skip and proceed to next attribute if this isn't valid
                attrs_bad.update({attr_name: attr_value})
                continue

            # Find out if value is already set to what we want. If yes, exclude
            # those attributes
            if data["Attributes"][attr_name] == attr_value:
                attrs_skipped.update({attr_name: attr_value})
            else:
                attrs_to_patch.update({attr_name: attr_value})

        warning = ""
        if attrs_bad:
            warning = f"Incorrect attributes {attrs_bad}"

        if not attrs_to_patch:
            return {
                "ret": True,
                "changed": False,
                "msg": "No changes made. Manager attributes already set.",
                "warning": warning,
            }

        payload = {"Attributes": attrs_to_patch}
        response = self.patch_request(f"{self.root_uri}{manager_uri}/{key}", payload)
        if response["ret"] is False:
            return response

        return {
            "ret": True,
            "changed": True,
            "msg": f"{command}: Modified Manager attributes {attrs_to_patch}",
            "warning": warning,
        }


CATEGORY_COMMANDS_ALL = {"Manager": ["SetManagerAttributes", "SetLifecycleControllerAttributes", "SetSystemAttributes"]}


# list of mutually exclusive commands for a category
CATEGORY_COMMANDS_MUTUALLY_EXCLUSIVE = {
    "Manager": [["SetManagerAttributes", "SetLifecycleControllerAttributes", "SetSystemAttributes"]]
}


def main():
    result = {}
    argument_spec = dict(
        category=dict(required=True),
        command=dict(required=True, type="list", elements="str"),
        baseuri=dict(required=True),
        username=dict(),
        password=dict(no_log=True),
        auth_token=dict(no_log=True),
        manager_attributes=dict(type="dict", default={}),
        timeout=dict(type="int", default=10),
        resource_id=dict(),
    )
    argument_spec.update(REDFISH_COMMON_ARGUMENT_SPEC)
    module = AnsibleModule(
        argument_spec,
        required_together=[
            ("username", "password"),
        ],
        required_one_of=[
            ("username", "auth_token"),
        ],
        mutually_exclusive=[
            ("username", "auth_token"),
        ],
        supports_check_mode=False,
    )

    category = module.params["category"]
    command_list = module.params["command"]

    # admin credentials used for authentication
    creds = {"user": module.params["username"], "pswd": module.params["password"], "token": module.params["auth_token"]}

    # timeout
    timeout = module.params["timeout"]

    # System, Manager or Chassis ID to modify
    resource_id = module.params["resource_id"]

    # Build root URI
    root_uri = f"https://{module.params['baseuri']}"
    rf_utils = IdracRedfishUtils(creds, root_uri, timeout, module, resource_id=resource_id, data_modification=True)

    # Check that Category is valid
    if category not in CATEGORY_COMMANDS_ALL:
        module.fail_json(msg=f"Invalid Category '{category}'. Valid Categories = {list(CATEGORY_COMMANDS_ALL.keys())}")

    # Check that all commands are valid
    for cmd in command_list:
        # Fail if even one command given is invalid
        if cmd not in CATEGORY_COMMANDS_ALL[category]:
            module.fail_json(msg=f"Invalid Command '{cmd}'. Valid Commands = {CATEGORY_COMMANDS_ALL[category]}")

    # check for mutually exclusive commands
    try:
        # check_mutually_exclusive accepts a single list or list of lists that
        # are groups of terms that should be mutually exclusive with one another
        # and checks that against a dictionary
        check_mutually_exclusive(CATEGORY_COMMANDS_MUTUALLY_EXCLUSIVE[category], dict.fromkeys(command_list, True))

    except TypeError as e:
        module.fail_json(msg=f"{e}")

    # Organize by Categories / Commands

    if category == "Manager":
        # execute only if we find a Manager resource
        result = rf_utils._find_managers_resource()
        if result["ret"] is False:
            module.fail_json(msg=result["msg"])

        for command in command_list:
            if command in ["SetManagerAttributes", "SetLifecycleControllerAttributes", "SetSystemAttributes"]:
                result = rf_utils.set_manager_attributes(command)

    # Return data back or fail with proper message
    if result["ret"] is True:
        if result.get("warning"):
            module.warn(result["warning"])

        module.exit_json(changed=result["changed"], msg=result["msg"])
    else:
        module.fail_json(msg=result["msg"])


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Jon Meran <jonathan.meran@sonos.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: batch_job_queue
version_added: 1.0.0
short_description: Manage AWS Batch Job Queues
description:
  - This module allows the management of AWS Batch Job Queues.
  - It is idempotent and supports "Check" mode.
  - Use module M(community.aws.batch_compute_environment) to manage the compute
    environment, M(community.aws.batch_job_queue) to manage job queues, M(community.aws.batch_job_definition) to manage job definitions.
  - Prior to release 5.0.0 this module was called C(community.aws.aws_batch_job_queue).
    The usage did not change.
author:
  - Jon Meran (@jonmer85)
options:
  job_queue_name:
    description:
      - The name for the job queue.
    required: true
    type: str
  state:
    description:
      - Describes the desired state.
    default: "present"
    choices: ["present", "absent"]
    type: str
  job_queue_state:
    description:
      - The state of the job queue. If the job queue state is ENABLED, it is able to accept jobs.
    default: "ENABLED"
    choices: ["ENABLED", "DISABLED"]
    type: str
  priority:
    description:
      - The priority of the job queue. Job queues with a higher priority (or a lower integer value for the priority
        parameter) are evaluated first when associated with same compute environment. Priority is determined in
        ascending order, for example, a job queue with a priority value of 1 is given scheduling preference over a job
        queue with a priority value of 10.
    required: true
    type: int
  compute_environment_order:
    description:
      - The set of compute environments mapped to a job queue and their order relative to each other. The job
        scheduler uses this parameter to determine which compute environment should execute a given job. Compute
        environments must be in the VALID state before you can associate them with a job queue. You can associate up to
        3 compute environments with a job queue.
    required: true
    type: list
    elements: dict
    suboptions:
      order:
        type: int
        description: The relative priority of the environment.
      compute_environment:
        type: str
        description: The name of the compute environment.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: My Batch Job Queue
  community.aws.batch_job_queue:
    job_queue_name: jobQueueName
    state: present
    region: us-east-1
    job_queue_state: ENABLED
    priority: 1
    compute_environment_order:
      - order: 1
        compute_environment: my_compute_env1
      - order: 2
        compute_environment: my_compute_env2
  register: batch_job_queue_action

- name: show results
  ansible.builtin.debug:
    var: batch_job_queue_action
"""

RETURN = r"""
---
output:
  description: "returns what action was taken, whether something was changed, invocation and response"
  returned: always
  sample:
    batch_job_queue_action: updated
    changed: false
    response:
      job_queue_arn: "arn:aws:batch:...."
      job_queue_name: <name>
      priority: 1
      state: DISABLED
      status: UPDATING
      status_reason: "JobQueue Healthy"
  type: dict
"""

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.batch import set_api_params

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule

# ---------------------------------------------------------------------------------------------------
#
#   Helper Functions & classes
#
# ---------------------------------------------------------------------------------------------------


def validate_params(module):
    """
    Performs basic parameter validation.

    :param module:
    """
    return


# ---------------------------------------------------------------------------------------------------
#
#   Batch Job Queue functions
#
# ---------------------------------------------------------------------------------------------------


def get_current_job_queue(module, client):
    try:
        environments = client.describe_job_queues(jobQueues=[module.params["job_queue_name"]])
        return environments["jobQueues"][0] if len(environments["jobQueues"]) > 0 else None
    except ClientError:
        return None


def create_job_queue(module, client):
    """
    Adds a Batch job queue

    :param module:
    :param client:
    :return:
    """

    changed = False

    # set API parameters
    params = ("job_queue_name", "priority")
    api_params = set_api_params(module, params)

    if module.params["job_queue_state"] is not None:
        api_params["state"] = module.params["job_queue_state"]

    api_params["computeEnvironmentOrder"] = get_compute_environment_order_list(module)

    try:
        if not module.check_mode:
            client.create_job_queue(**api_params)
        changed = True
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e, msg="Error creating compute environment")

    return changed


def get_compute_environment_order_list(module):
    compute_environment_order_list = []
    for ceo in module.params["compute_environment_order"]:
        compute_environment_order_list.append(dict(order=ceo["order"], computeEnvironment=ceo["compute_environment"]))
    return compute_environment_order_list


def remove_job_queue(module, client):
    """
    Remove a Batch job queue

    :param module:
    :param client:
    :return:
    """

    changed = False

    # set API parameters
    api_params = {"jobQueue": module.params["job_queue_name"]}

    try:
        if not module.check_mode:
            client.delete_job_queue(**api_params)
        changed = True
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e, msg="Error removing job queue")
    return changed


def manage_state(module, client):
    changed = False
    current_state = "absent"
    state = module.params["state"]
    job_queue_state = module.params["job_queue_state"]
    job_queue_name = module.params["job_queue_name"]
    priority = module.params["priority"]
    action_taken = "none"
    response = None

    check_mode = module.check_mode

    # check if the job queue exists
    current_job_queue = get_current_job_queue(module, client)
    if current_job_queue:
        current_state = "present"

    if state == "present":
        if current_state == "present":
            updates = False
            # Update Batch Job Queue configuration
            job_kwargs = {"jobQueue": job_queue_name}

            # Update configuration if needed
            if job_queue_state and current_job_queue["state"] != job_queue_state:
                job_kwargs.update({"state": job_queue_state})
                updates = True
            if priority is not None and current_job_queue["priority"] != priority:
                job_kwargs.update({"priority": priority})
                updates = True

            new_compute_environment_order_list = get_compute_environment_order_list(module)
            if new_compute_environment_order_list != current_job_queue["computeEnvironmentOrder"]:
                job_kwargs["computeEnvironmentOrder"] = new_compute_environment_order_list
                updates = True

            if updates:
                try:
                    if not check_mode:
                        client.update_job_queue(**job_kwargs)
                    changed = True
                    action_taken = "updated"
                except (BotoCoreError, ClientError) as e:
                    module.fail_json_aws(e, msg="Unable to update job queue")

        else:
            # Create Job Queue
            changed = create_job_queue(module, client)
            action_taken = "added"

        # Describe job queue
        response = get_current_job_queue(module, client)
        if not response:
            module.fail_json(msg="Unable to get job queue information after creating/updating")
    else:
        if current_state == "present":
            # remove the Job Queue
            changed = remove_job_queue(module, client)
            action_taken = "deleted"
    return dict(changed=changed, batch_job_queue_action=action_taken, response=response)


# ---------------------------------------------------------------------------------------------------
#
#   MAIN
#
# ---------------------------------------------------------------------------------------------------


def main():
    """
    Main entry point.

    :return dict: changed, batch_job_queue_action, response
    """

    argument_spec = dict(
        state=dict(required=False, default="present", choices=["present", "absent"]),
        job_queue_name=dict(required=True),
        job_queue_state=dict(required=False, default="ENABLED", choices=["ENABLED", "DISABLED"]),
        priority=dict(type="int", required=True),
        compute_environment_order=dict(type="list", required=True, elements="dict"),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    client = module.client("batch")

    validate_params(module)

    results = manage_state(module, client)

    module.exit_json(**camel_dict_to_snake_dict(results))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
from ansible.plugins.action import ActionBase
try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator, )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.meraki.plugins.plugin_utils.meraki import (
    MERAKI,
    meraki_argument_spec,
    meraki_compare_equality2,
    get_dict_result,
)
from ansible_collections.cisco.meraki.plugins.plugin_utils.exceptions import (
    InconsistentParameters,
)

# Get common arguments specification
argument_spec = meraki_argument_spec()
# Add arguments specific for this module
argument_spec.update(dict(
    state=dict(type="str", default="present", choices=["present"]),
    enabled=dict(type="bool"),
    deviceTypePolicies=dict(type="list"),
    networkId=dict(type="str"),
    number=dict(type="str"),
))

required_if = [
    ("state", "present", ["networkId", "number"], True),
]
required_one_of = []
mutually_exclusive = []
required_together = []


class NetworksWirelessSsidsDeviceTypeGroupPolicies(object):
    def __init__(self, params, meraki):
        self.meraki = meraki
        self.new_object = dict(
            enabled=params.get("enabled"),
            deviceTypePolicies=params.get("deviceTypePolicies"),
            networkId=params.get("networkId"),
            number=params.get("number"),
        )

    def get_all_params(self, name=None, id=None):
        new_object_params = {}
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        if self.new_object.get('number') is not None or self.new_object.get(
                'number') is not None:
            new_object_params['number'] = self.new_object.get('number')
        return new_object_params

    def update_all_params(self):
        new_object_params = {}
        if self.new_object.get('enabled') is not None or self.new_object.get(
                'enabled') is not None:
            new_object_params['enabled'] = self.new_object.get('enabled')
        if self.new_object.get('deviceTypePolicies') is not None or self.new_object.get(
                'device_type_policies') is not None:
            new_object_params['deviceTypePolicies'] = self.new_object.get(
                'deviceTypePolicies') or self.new_object.get('device_type_policies')
        if self.new_object.get('networkId') is not None or self.new_object.get(
                'network_id') is not None:
            new_object_params['networkId'] = self.new_object.get(
                'networkId') or self.new_object.get('network_id')
        if self.new_object.get('number') is not None or self.new_object.get(
                'number') is not None:
            new_object_params['number'] = self.new_object.get('number') or \
                self.new_object.get('number')
        return new_object_params

    def get_object_by_name(self, name):
        result = None
        # NOTE: Does not have a get by name method, using get all
        try:
            items = self.meraki.exec_meraki(
                family="wireless",
                function="getNetworkWirelessSsidDeviceTypeGroupPolicies",
                params=self.get_all_params(name=name),
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = get_dict_result(items, 'name', name)
            if result is None:
                result = items
        except Exception as e:
            print("Error: ", e)
            result = None
        return result

    def get_object_by_id(self, id):
        result = None
        # NOTE: Does not have a get by id method or it is in another action
        return result

    def exists(self):
        prev_obj = None
        id_exists = False
        name_exists = False
        o_id = self.new_object.get(
            "networkId") or self.new_object.get("network_id")
        name = self.new_object.get("name")
        if o_id:
            prev_obj = self.get_object_by_name(o_id)
            id_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if not id_exists and name:
            prev_obj = self.get_object_by_name(name)
            name_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if name_exists:
            _id = prev_obj.get("id")
            if id_exists and name_exists and o_id != _id:
                raise InconsistentParameters(
                    "The 'id' and 'name' params don't refer to the same object")
            if _id:
                self.new_object.update(dict(id=_id))
        it_exists = prev_obj is not None and isinstance(prev_obj, dict)
        return (it_exists, prev_obj)

    def requires_update(self, current_obj):
        requested_obj = self.new_object

        obj_params = [
            ("enabled", "enabled"),
            ("deviceTypePolicies", "deviceTypePolicies"), ("number", "number"),
        ]
        # Method 1. Params present in request (Ansible) obj are the same as the current (ISE) params
        # If any does not have eq params, it requires update
        return any(
            not meraki_compare_equality2(
                current_obj.get(meraki_param),
                requested_obj.get(ansible_param)) for (
                meraki_param,
                ansible_param) in obj_params)

    def update(self):
        id = self.new_object.get("id")
        name = self.new_object.get("name")
        result = None
        result = self.meraki.exec_meraki(
            family="wireless",
            function="updateNetworkWirelessSsidDeviceTypeGroupPolicies",
            params=self.update_all_params(),
            op_modifies=True,
        )
        return result


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail(
                "ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'")
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        meraki = MERAKI(self._task.args)
        obj = NetworksWirelessSsidsDeviceTypeGroupPolicies(
            self._task.args, meraki)

        state = self._task.args.get("state")

        response = None
        if state == "present":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                if obj.requires_update(prev_obj):
                    response = obj.update()
                    meraki.object_updated()
                else:
                    response = prev_obj
                    meraki.object_already_present()
            else:
                meraki.fail_json(
                    "Object does not exists, plugin only has update")

        self._result.update(dict(meraki_response=response))
        self._result.update(meraki.exit_json())
        return self._result



#!/usr/bin/python
# Copyright (c) 2020, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: openssl_privatekey_pipe
short_description: Generate OpenSSL private keys without disk access
version_added: 1.3.0
description:
  - This module allows one to (re)generate OpenSSL private keys without disk access.
  - This allows to read and write keys to vaults without having to write intermediate versions to disk.
  - Make sure to not write the result of this module into logs or to the console, as it contains private key data! Use the
    C(no_log) task option to be sure.
  - Note that this module is implemented as an L(action plugin,https://docs.ansible.com/ansible/latest/plugins/action.html)
    and will always be executed on the controller.
author:
  - Yanis Guenane (@Spredzy)
  - Felix Fontein (@felixfontein)
extends_documentation_fragment:
  - community.crypto._attributes
  - community.crypto._attributes.flow
  - community.crypto._module_privatekey
attributes:
  action:
    support: full
  async:
    support: none
    details:
      - This action runs completely on the controller.
  check_mode:
    support: full
    details:
      - Since community.crypto 3.0.0, the module ignores check mode and always behaves as if check mode is not active.
options:
  content:
    description:
      - The current private key data.
      - Needed for idempotency. If not provided, the module will always return a change, and all idempotence-related options
        are ignored.
    type: str
  content_base64:
    description:
      - Set to V(true) if the content is base64 encoded.
    type: bool
    default: false
  return_current_key:
    description:
      - Set to V(true) to return the current private key when the module did not generate a new one.
      - Note that in case of check mode, when this option is not set to V(true), the module always returns the current key
        (if it was provided) and Ansible will replace it by C(VALUE_SPECIFIED_IN_NO_LOG_PARAMETER).
    type: bool
    default: false
  regenerate:
    description:
      - Allows to configure in which situations the module is allowed to regenerate private keys. The module will always generate
        a new key if the destination file does not exist.
      - By default, the key will be regenerated when it does not match the module's options, except when the key cannot be
        read or the passphrase does not match. Please note that this B(changed) for Ansible 2.10. For Ansible 2.9, the behavior
        was as if V(full_idempotence) is specified.
      - If set to V(never), the module will fail if the key cannot be read or the passphrase is not matching, and will never
        regenerate an existing key.
      - If set to V(fail), the module will fail if the key does not correspond to the module's options.
      - If set to V(partial_idempotence), the key will be regenerated if it does not conform to the module's options. The
        key is B(not) regenerated if it cannot be read (broken file), the key is protected by an unknown passphrase, or when
        they key is not protected by a passphrase, but a passphrase is specified.
      - If set to V(full_idempotence), the key will be regenerated if it does not conform to the module's options. This is
        also the case if the key cannot be read (broken file), the key is protected by an unknown passphrase, or when they
        key is not protected by a passphrase, but a passphrase is specified. Make sure you have a B(backup) when using this
        option!
      - If set to V(always), the module will always regenerate the key.
      - Note that if O(format_mismatch) is set to V(convert) and everything matches except the format, the key will always
        be converted, except if O(regenerate) is set to V(always).
seealso:
  - module: community.crypto.openssl_privatekey
  - module: community.crypto.openssl_privatekey_info
"""

EXAMPLES = r"""
---
- name: Generate an OpenSSL private key with the default values (4096 bits, RSA)
  community.crypto.openssl_privatekey_pipe:
  register: output
  no_log: true # make sure that private key data is not accidentally revealed in logs!
- name: Show generated key
  ansible.builtin.debug:
    msg: "{{ output.privatekey }}"
  # DO NOT OUTPUT KEY MATERIAL TO CONSOLE OR LOGS IN PRODUCTION!


# The following example needs CNCF SOPS (https://github.com/getsops/sops) set up and
# the community.sops collection installed. See also
# https://docs.ansible.com/ansible/latest/collections/community/sops/docsite/guide.html

- name: Generate or update a CNCF SOPS encrypted key
  block:
    - name: Update SOPS-encrypted key with the community.sops collection
      community.crypto.openssl_privatekey_pipe:
        content: "{{ lookup('community.sops.sops', 'private_key.pem.sops') }}"
        size: 2048
      register: output
      no_log: true # make sure that private key data is not accidentally revealed in logs!

    - name: Update encrypted key when openssl_privatekey_pipe reported a change
      community.sops.sops_encrypt:
        path: private_key.pem.sops
        content_text: "{{ output.privatekey }}"
      when: output is changed
  always:
    - name: Make sure that output (which contains the private key) is overwritten
      ansible.builtin.set_fact:
        output: ''
"""

RETURN = r"""
size:
  description: Size (in bits) of the TLS/SSL private key.
  returned: changed or success
  type: int
  sample: 4096
type:
  description: Algorithm used to generate the TLS/SSL private key.
  returned: changed or success
  type: str
  sample: RSA
curve:
  description: Elliptic curve used to generate the TLS/SSL private key.
  returned: changed or success, and O(type) is V(ECC)
  type: str
  sample: secp256r1
fingerprint:
  description:
    - The fingerprint of the public key. Fingerprint will be generated for each C(hashlib.algorithms) available.
  returned: changed or success
  type: dict
  sample:
    md5: "84:75:71:72:8d:04:b5:6c:4d:37:6d:66:83:f5:4c:29"
    sha1: "51:cc:7c:68:5d:eb:41:43:88:7e:1a:ae:c7:f8:24:72:ee:71:f6:10"
    sha224: "b1:19:a6:6c:14:ac:33:1d:ed:18:50:d3:06:5c:b2:32:91:f1:f1:52:8c:cb:d5:75:e9:f5:9b:46"
    sha256: "41:ab:c7:cb:d5:5f:30:60:46:99:ac:d4:00:70:cf:a1:76:4f:24:5d:10:24:57:5d:51:6e:09:97:df:2f:de:c7"
    sha384: "85:39:50:4e:de:d9:19:33:40:70:ae:10:ab:59:24:19:51:c3:a2:e4:0b:1c:b1:6e:dd:b3:0c:d9:9e:6a:46:af:da:18:f8:ef:ae:2e:c0:9a:75:2c:9b:b3:0f:3a:5f:3d"
    sha512: "fd:ed:5e:39:48:5f:9f:fe:7f:25:06:3f:79:08:cd:ee:a5:e7:b3:3d:13:82:87:1f:84:e1:f5:c7:28:77:53:94:86:56:38:69:f0:d9:35:22:01:1e:a6:60:...:0f:9b"
privatekey:
  description:
    - The generated private key's content.
    - Please note that if the result is not changed, the current private key will only be returned if the O(return_current_key)
      option is set to V(true).
    - Will be Base64-encoded if the key is in raw format.
  returned: changed, or O(return_current_key) is V(true)
  type: str
"""



#!/usr/bin/python
#
# Copyright Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: campfire
short_description: Send a message to Campfire
description:
  - Send a message to Campfire.
  - Messages with newlines result in a "Paste" message being sent.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  subscription:
    type: str
    description:
      - The subscription name to use.
    required: true
  token:
    type: str
    description:
      - API token.
    required: true
  room:
    type: str
    description:
      - Room number to which the message should be sent.
    required: true
  msg:
    type: str
    description:
      - The message body.
    required: true
  notify:
    type: str
    description:
      - Send a notification sound before the message.
    choices:
      - 56k
      - bell
      - bezos
      - bueller
      - clowntown
      - cottoneyejoe
      - crickets
      - dadgummit
      - dangerzone
      - danielsan
      - deeper
      - drama
      - greatjob
      - greyjoy
      - guarantee
      - heygirl
      - horn
      - horror
      - inconceivable
      - live
      - loggins
      - makeitso
      - noooo
      - nyan
      - ohmy
      - ohyeah
      - pushit
      - rimshot
      - rollout
      - rumble
      - sax
      - secret
      - sexyback
      - story
      - tada
      - tmyk
      - trololo
      - trombone
      - unix
      - vuvuzela
      - what
      - whoomp
      - yeah
      - yodel

# informational: requirements for nodes
requirements: []
author: "Adam Garside (@fabulops)"
"""

EXAMPLES = r"""
- name: Send a message to Campfire
  community.general.campfire:
    subscription: foo
    token: 12345
    room: 123
    msg: Task completed.

- name: Send a message to Campfire
  community.general.campfire:
    subscription: foo
    token: 12345
    room: 123
    notify: loggins
    msg: Task completed ... with feeling.
"""

from html import escape as html_escape

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url


def main():
    module = AnsibleModule(
        argument_spec=dict(
            subscription=dict(required=True),
            token=dict(required=True, no_log=True),
            room=dict(required=True),
            msg=dict(required=True),
            notify=dict(
                choices=[
                    "56k",
                    "bell",
                    "bezos",
                    "bueller",
                    "clowntown",
                    "cottoneyejoe",
                    "crickets",
                    "dadgummit",
                    "dangerzone",
                    "danielsan",
                    "deeper",
                    "drama",
                    "greatjob",
                    "greyjoy",
                    "guarantee",
                    "heygirl",
                    "horn",
                    "horror",
                    "inconceivable",
                    "live",
                    "loggins",
                    "makeitso",
                    "noooo",
                    "nyan",
                    "ohmy",
                    "ohyeah",
                    "pushit",
                    "rimshot",
                    "rollout",
                    "rumble",
                    "sax",
                    "secret",
                    "sexyback",
                    "story",
                    "tada",
                    "tmyk",
                    "trololo",
                    "trombone",
                    "unix",
                    "vuvuzela",
                    "what",
                    "whoomp",
                    "yeah",
                    "yodel",
                ]
            ),
        ),
        supports_check_mode=False,
    )

    subscription = module.params["subscription"]
    token = module.params["token"]
    room = module.params["room"]
    msg = module.params["msg"]
    notify = module.params["notify"]

    URI = f"https://{subscription}.campfirenow.com"
    AGENT = "Ansible/1.2"

    # Hack to add basic auth username and password the way fetch_url expects
    module.params["url_username"] = token
    module.params["url_password"] = "X"

    target_url = f"{URI}/room/{room}/speak.xml"
    headers = {"Content-Type": "application/xml", "User-agent": AGENT}

    # Send some audible notification if requested
    if notify:
        response, info = fetch_url(
            module,
            target_url,
            data=f"<message><type>SoundMessage</type><body>{html_escape(notify)}</body></message>",
            headers=headers,
        )
        if info["status"] not in [200, 201]:
            module.fail_json(
                msg=f"unable to send msg: '{notify}', campfire api returned error code: '{info['status']}'"
            )

    # Send the message
    response, info = fetch_url(
        module, target_url, data=f"<message><body>{html_escape(msg)}</body></message>", headers=headers
    )
    if info["status"] not in [200, 201]:
        module.fail_json(msg=f"unable to send msg: '{msg}', campfire api returned error code: '{info['status']}'")

    module.exit_json(changed=True, room=room, msg=msg, notify=notify)


if __name__ == "__main__":
    main()



# Copyright (c) 2021 Felix Fontein
# Copyright (c) 2020 Markus Bergholz <markuman+spambelongstogoogle@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this plugin util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import abc
import typing as t
from collections.abc import Sequence

from ansible.errors import AnsibleError
from ansible.plugins.inventory import BaseInventoryPlugin
from ansible.template import Templar
from ansible.utils.display import Display
from ansible_collections.community.library_inventory_filtering_v1.plugins.plugin_utils.inventory_filter import (
    filter_host,
    parse_filters,
)

from ansible_collections.community.dns.plugins.module_utils._conversion.base import (
    DNSConversionError,
)
from ansible_collections.community.dns.plugins.module_utils._conversion.converter import (
    RecordConverter,
)
from ansible_collections.community.dns.plugins.module_utils._provider import (
    ProviderInformation,
    ensure_type,
)
from ansible_collections.community.dns.plugins.module_utils._zone_record_api import (
    DNSAPIAuthenticationError,
    DNSAPIError,
    ZoneRecordAPI,
)
from ansible_collections.community.dns.plugins.module_utils._zone_record_set_api import (
    ZoneRecordSetAPI,
)
from ansible_collections.community.dns.plugins.plugin_utils._unsafe import make_unsafe

display = Display()


class RecordsInventoryModule(BaseInventoryPlugin, metaclass=abc.ABCMeta):
    VALID_ENDINGS = ("dns.yaml", "dns.yml")
    NAME: str

    @abc.abstractmethod
    def setup_api(self) -> tuple[ProviderInformation, ZoneRecordAPI | ZoneRecordSetAPI]:
        """
        This function needs to create a provider information and an API object.
        It can indicate errors by raising DNSAPIError.
        """

    def verify_file(self, path: str) -> bool:
        if super().verify_file(path):
            if path.endswith(self.VALID_ENDINGS):
                return True
            endings = " or ".join([f"'{ending}'" for ending in self.VALID_ENDINGS])
            display.debug(f"{self.NAME} inventory filename must end with {endings}")
        return False

    def parse(
        self, inventory: t.Any, loader: t.Any, path: str, cache: bool = False
    ) -> None:
        super().parse(inventory, loader, path, cache)

        self._read_config_data(path)

        self.templar = Templar(loader=loader)

        try:
            provider_information, api = self.setup_api()

            record_converter = RecordConverter(provider_information, self)
            record_converter.emit_deprecations(display.deprecated)

            zone_name = self.get_option("zone_name")
            if self.templar.is_template(zone_name):
                zone_name = self.templar.template(variable=zone_name)
            zone_id = self.get_option("zone_id")
            if zone_id is not None:
                if self.templar.is_template(zone_id):
                    zone_id = self.templar.template(variable=zone_id)
                # For templating, we need to make the zone_id type 'string' or 'raw'.
                # This converts the value to its proper type expected by the API.
                zone_id_type = provider_information.get_record_id_type()
                try:
                    zone_id = ensure_type(zone_id, zone_id_type)
                except TypeError as exc:
                    raise AnsibleError(
                        f"Error while ensuring that zone_id is of type {zone_id_type}: {exc}"
                    ) from exc

            if isinstance(api, ZoneRecordAPI):
                if zone_name is not None:
                    zone_with_records = api.get_zone_with_records_by_name(zone_name)
                elif zone_id is not None:
                    zone_with_records = api.get_zone_with_records_by_id(zone_id)
                else:
                    raise AnsibleError(
                        "One of zone_name and zone_id must be specified!"
                    )

                if zone_with_records is None:
                    raise AnsibleError("Zone does not exist")

                zone = zone_with_records.zone
                records = zone_with_records.records
            else:
                if zone_name is not None:
                    zone_with_record_sets = api.get_zone_with_record_sets_by_name(
                        zone_name
                    )
                elif zone_id is not None:
                    zone_with_record_sets = api.get_zone_with_record_sets_by_id(zone_id)
                else:
                    raise AnsibleError(
                        "One of zone_name and zone_id must be specified!"
                    )

                if zone_with_record_sets is None:
                    raise AnsibleError("Zone does not exist")

                zone = zone_with_record_sets.zone
                records = []
                for record_set in zone_with_record_sets.record_sets:
                    records.extend(record_set.records)

            record_converter.process_multiple_from_api(records)
            record_converter.process_multiple_to_user(records)

        except DNSConversionError as e:
            raise AnsibleError(
                f"Error while converting DNS values: {e.error_message}"
            ) from e
        except DNSAPIAuthenticationError as e:
            raise AnsibleError(f"Cannot authenticate: {e}") from e
        except DNSAPIError as e:
            raise AnsibleError(f"Error: {e}") from e

        simple_filters = self.get_option("simple_filters")
        filters = parse_filters(self.get_option("filters"))

        filter_types = simple_filters.get("type") or ["A", "AAAA", "CNAME"]
        if not isinstance(filter_types, Sequence) or isinstance(
            filter_types, (str, bytes)
        ):
            filter_types = [filter_types]

        if self.inventory is None:  # pragma: no cover
            raise AssertionError(  # pragma: no cover
                "Inventory must not be None in parse()"
            )
        for record in records:
            if record.type in filter_types:
                name = zone.name
                if record.prefix:
                    name = f"{record.prefix}.{name}"
                facts = {
                    "ansible_host": make_unsafe(record.target),
                }
                if not filter_host(self, name, facts, filters):
                    continue

                self.inventory.add_host(name)
                for key, value in facts.items():
                    self.inventory.set_variable(name, key, value)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_central_license_facts
short_description: Get central-license objects facts on Checkpoint over Web Services API
description:
  - Get central-license objects facts on Checkpoint devices.
  - All operations are performed over Web Services API.
  - This module handles both operations, get a specific object and get several objects,
    For getting a specific object use the parameter 'signature'.
  - Available from R81.20 JHF management version.
version_added: "5.2.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  signature:
    description:
      - The license's signature. This parameter is relevant only for getting a specific object.
    type: str
extends_documentation_fragment: checkpoint_facts
"""

EXAMPLES = """
- name: show-central-license
  cp_mgmt_central_license_facts:
    signature: dLLLLL-WWWWWW-ZZZZZZ-QQQQQQ

- name: show-central-licenses
  cp_mgmt_show_central_licenses:
"""

RETURN = """
ansible_facts:
  description: The checkpoint object facts.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_facts,
    api_call_facts,
)


def main():
    argument_spec = dict(
        signature=dict(type='str')
    )
    argument_spec.update(checkpoint_argument_spec_for_facts)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    api_call_object = "central-license"
    api_call_object_plural_version = "central-licenses"

    result = api_call_facts(module, api_call_object, api_call_object_plural_version)
    module.exit_json(ansible_facts=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_info
short_description: Information module for networks
description:
  - Get all networks.
  - Get networks by id.
  - List the networks that the user has privileges on in an organization.
  - Return a network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  networkId:
    description:
      - NetworkId path parameter. Network ID.
    type: str
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  configTemplateId:
    description:
      - >
        ConfigTemplateId query parameter. An optional parameter that is the ID of
        a config template. Will return all networks bound to that template.
    type: str
  isBoundToConfigTemplate:
    description:
      - >
        IsBoundToConfigTemplate query parameter. An optional parameter to filter config
        template bound networks. If configTemplateId is set, this cannot be false.
    type: bool
  tags:
    description:
      - >
        Tags query parameter. An optional parameter to filter networks by tags. The
        filtering is case-sensitive. If tags are included, 'tagsFilterType' should
        also be included (see below).
    elements: str
    type: list
  tagsFilterType:
    description:
      - >
        TagsFilterType query parameter. An optional parameter of value 'withAnyTags'
        or 'withAllTags' to indicate whether to return networks which contain ANY
        or ALL of the included tags. If no type is included, 'withAnyTags' will be
        selected.
    type: str
  productTypes:
    description:
      - >
        ProductTypes query parameter. An optional parameter to filter networks by
        product type. Results will have at least one of the included product types.
    elements: str
    type: list
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 100000. Default is 1000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks getNetwork
    description: Complete reference of the getNetwork API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-network
  - name: Cisco Meraki documentation for networks getOrganizationNetworks
    description: Complete reference of the getOrganizationNetworks API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-networks
notes:
  - SDK Method used are
    networks.Networks.get_network,
    networks.Networks.get_organization_networks,
  - Paths used are
    get /networks/{networkId},
    get /organizations/{organizationId}/networks,
"""

EXAMPLES = r"""
- name: Get all networks
  cisco.meraki.networks_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    configTemplateId: string
    isBoundToConfigTemplate: true
    tags: []
    tagsFilterType: string
    productTypes: []
    perPage: 0
    startingAfter: string
    endingBefore: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
- name: Get networks by id
  cisco.meraki.networks_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "organizationId": "string",
      "name": "string",
      "productTypes": [
        "string"
      ],
      "timeZone": "string",
      "tags": [
        "string"
      ],
      "enrollmentString": "string",
      "url": "string",
      "notes": "string",
      "isBoundToConfigTemplate": true
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: waf_info
short_description: Retrieve information for WAF ACLs, Rules, Conditions and Filters
version_added: 1.0.0

description:
  - Retrieve information for WAF ACLs, Rules, Conditions and Filters.
  - Prior to release 5.0.0 this module was called C(community.aws.aws_waf_info).
    The usage did not change.

author:
  - Mike Mochan (@mmochan)
  - Will Thames (@willthames)

deprecated:
  removed_in: 12.0.0
  why: Amazon has retired the "WAF Classic" service.
  alternative: Please use the "AWS WAF (WAFv2)" service and modules instead.

options:
  name:
    description:
      - The name of a Web Application Firewall.
    type: str
  waf_regional:
    description: Whether to use the C(waf-regional) module.
    default: false
    required: false
    type: bool

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: obtain all WAF information
  community.aws.waf_info:

- name: obtain all information for a single WAF
  community.aws.waf_info:
    name: test_waf

- name: obtain all information for a single WAF Regional
  community.aws.waf_info:
    name: test_waf
    waf_regional: true
"""

RETURN = r"""
wafs:
  description: The WAFs that match the passed arguments.
  returned: success
  type: complex
  contains:
    name:
      description: A friendly name or description of the WebACL.
      returned: always
      type: str
      sample: test_waf
    default_action:
      description: The action to perform if none of the Rules contained in the WebACL match.
      returned: always
      type: int
      sample: BLOCK
    metric_name:
      description: A friendly name or description for the metrics for this WebACL.
      returned: always
      type: str
      sample: test_waf_metric
    rules:
      description: An array that contains the action for each Rule in a WebACL , the priority of the Rule.
      returned: always
      type: complex
      contains:
        action:
          description: The action to perform if the Rule matches.
          returned: always
          type: str
          sample: BLOCK
        metric_name:
          description: A friendly name or description for the metrics for this Rule.
          returned: always
          type: str
          sample: ipblockrule
        name:
          description: A friendly name or description of the Rule.
          returned: always
          type: str
          sample: ip_block_rule
        predicates:
          description: The Predicates list contains a Predicate for each
            ByteMatchSet, IPSet, SizeConstraintSet, SqlInjectionMatchSet or XssMatchSet
            object in a Rule.
          returned: always
          type: list
          sample:
            [
              {
                "byte_match_set_id": "47b822b5-abcd-1234-faaf-1234567890",
                "byte_match_tuples": [
                  {
                    "field_to_match": {
                      "type": "QUERY_STRING"
                    },
                    "positional_constraint": "STARTS_WITH",
                    "target_string": "bobbins",
                    "text_transformation": "NONE"
                  }
                ],
                "name": "bobbins",
                "negated": false,
                "type": "ByteMatch"
              }
            ]
"""

from ansible_collections.amazon.aws.plugins.module_utils.waf import get_web_acl
from ansible_collections.amazon.aws.plugins.module_utils.waf import list_web_acls

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def main():
    argument_spec = dict(
        name=dict(required=False),
        waf_regional=dict(type="bool", default=False),
    )
    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    resource = "waf" if not module.params["waf_regional"] else "waf-regional"
    client = module.client(resource)
    web_acls = list_web_acls(client, module)
    name = module.params["name"]
    if name:
        web_acls = [web_acl for web_acl in web_acls if web_acl["Name"] == name]
        if not web_acls:
            module.fail_json(msg=f"WAF named {name} not found")
    module.exit_json(wafs=[get_web_acl(client, module, web_acl["WebACLId"]) for web_acl in web_acls])


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_identity_provider
short_description: Manages identity-provider objects on Checkpoint over Web Services API
description:
  - Manages identity-provider objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R82 JHF management version.
version_added: "6.5.0"
author: "Dor Berenstein (@chkp-dorbe)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  usage:
    description:
      - Usage of Identity Provider.
    type: str
    choices: ['managing_administrator_access', 'gateway_policy_and_logs']
  gateway:
    description:
      - Gateway for the SAML Identity Provider usage.
        Identified by name or UID. <font color="red">Required only when</font> 'usage' is set to 'gateway_policy_and_logs'.
    type: str
  service:
    description:
      - Service for the selected gateway. <font color="red">Required only when</font> 'usage' is set to 'gateway_policy_and_logs'.
    type: str
    choices: ['none', 'identity awareness', 'mobile access', 'vpn']
  data_receiving:
    description:
      - Data receiving method from the SAML Identity Provider.
    type: str
    choices: ['metadata_file', 'manually']
  received_identifier:
    description:
      - Received Identifier (Entity ID) based on the provider data. <font color="red">Required only when</font> 'data-receiving' is set to 'manually'.
    type: str
  login_url:
    description:
      - Login URL based on the provider data. <font color="red">Required only when</font> 'data-receiving' is set to 'manually'.
    type: str
  base64_metadata_file:
    description:
      - Metadata file encoded in base64 based on the provider data. <font color="red">Required only when</font> 'data-receiving' is set to 'metadata_file'.
    type: str
  base64_certificate:
    description:
      - Certificate file encoded in base64 based on provider data. <font color="red">Required only when</font> 'data-receiving' is set to 'manually'.
    type: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
    type: list
    elements: str
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-identity-provider
  cp_mgmt_identity_provider:
    base64_certificate: |
        LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0tCk1JSUM4RENDQWRpZ0F3SUJBZ0lRUTBWZVpLdVBLb0pQUWhaNGhDaWRzREFOQmdrcWhraUc5dzBCQVFzRkFEQTBNVEl3TUFZRFZRUURFeWxOYVdOeWIzTn
        ablFnUVhwMWNtVWdSbVZrWlhKaGRHVmtJRk5UVHlCRFpYSjBhV1pwWTJGMFpUQWVGdzB4T0RBME1UVXhNVEl6TXpOYUZ3MHlNVEEwTVRVeE1USXpNek5hTURReE1qQXdCZ05WQkFNVEtVMXBZM0p2YzI5b
        RDQkJlblZ5WlNCR1pXUmxjbUYwWldRZ1UxTlBJRU5sY25ScFptbGpZWFJsTUlJQklqQU5CZ2txaGtpRzl3MEJBUUVGQUFPQ0FROEFNSUlCQ2dLQ0FRRUE0VXVqYUd0OFhaODl2dXZ5a3lRVzVYb24vOFIv
        VB0ejRhYjBNM3RNVXZHWHozVXh0V1pTRStUR1hydjN3VHRLMCs4RmtNeXVKYUhGSXBLLzRVREZpRk1yQmxzR0Z1dmtTc1p5VjIzZlNGN3paaXlUWTZUN0EwcCtnczUwNVhEOUdBYjlWYmR3R0cwK0tDVnl
        c1ZRZ1YySXdKZ2l5aHF3RUNvY3dCcmFuN251SytURU5EMmwyZjlZcng1b1JNRU56NzB3bHlIMzZPWkJtdDBrNTk4L1doMEhEWUxaZW8wZHlTV3JOd3dlWXZTeEU4L01kbTQzWEV1U3pialR6ZnNNMHZVUn
        GQlNyVUxFYURPMS9JUDJVcjdCc2dId1JJL3hmb3FJbUsxS2twVXEwQWxjVEFpM3YxdTl6Qy9xTGdQd0F5UUl2dzlVQ3NpcnJQQTBZMFlPaFFJREFRQUJNQTBHQ1NxR1NJYjNEUUVCQ3dVQUE0SUJBUURja
        9qZEd6L0FJQ2pqTTBaN21ZbGdQNXpic2FRNWRDMmNqZjRESnFta21zV3VmUnBDNHNic3VoODcwY0NCS2N1dmgrb0dpekJRSHJQbTRUaEl2ZklsS0w4WGpMQVhiRnVSUG9IQWcwOHNMWGR2UFRCVE52REYx
        WhvcU5zMmo2ZUZxL2ROdXF2ZUJIcjVENXRLblYyWEJHRUhFOVJFOVdUa1pRT2MwaEhtQ3dNbWNZb3JYRzhCa3l1OXFwNXhyMDZMQ0htMnJLcnI2ZENRVldBV0R0MzhiS2t5STRobTVSNTVCclR5UldSdzI
        RS9YaFEwVksva1FJYW9GZ0hvaWo0ekg5bmxlZnZMbmhaZDVPRzROL29OS2pBKy9LbkVqaTdPQXhKWVNaR1FmRjU0R1AwQTE4VnF1NVVGaFBKMUZFQXZ5YjR0QnZtTzM1NFFVUys5UTY2agotLS0tLUVORC
        DRVJUSUZJQ0FURS0tLS0t
    data_receiving: manually
    login_url: https://login.checkpointonline.com/621ac12d-4afb-479c-9c14-13e7b743cd07/saml2
    name: TestIdp2
    received_identifier: https://sts.checkpoint.net/621ac12d-4afb-479c-9c14-13e7b743cd07/
    state: present
    usage: managing_administrator_access

- name: set-identity-provider
  cp_mgmt_identity_provider:
    base64_certificate: |
        LS0tLS1CRUdJTiBDRVJUSUZJQ0FURS0tLS0tCk1JSUM4RENDQWRpZ0F3SUJBZ0lRUTBWZVpLdVBLb0pQUWhaNGhDaWRzREFOQmdrcWhraUc5dzBCQVFzRkFEQTBNVEl3TUFZRFZRUURFeWxOYVdOeWIzTn
        ablFnUVhwMWNtVWdSbVZrWlhKaGRHVmtJRk5UVHlCRFpYSjBhV1pwWTJGMFpUQWVGdzB4T0RBME1UVXhNVEl6TXpOYUZ3MHlNVEEwTVRVeE1USXpNek5hTURReE1qQXdCZ05WQkFNVEtVMXBZM0p2YzI5b
        RDQkJlblZ5WlNCR1pXUmxjbUYwWldRZ1UxTlBJRU5sY25ScFptbGpZWFJsTUlJQklqQU5CZ2txaGtpRzl3MEJBUUVGQUFPQ0FROEFNSUlCQ2dLQ0FRRUE0VXVqYUd0OFhaODl2dXZ5a3lRVzVYb24vOFIv
        VB0ejRhYjBNM3RNVXZHWHozVXh0V1pTRStUR1hydjN3VHRLMCs4RmtNeXVKYUhGSXBLLzRVREZpRk1yQmxzR0Z1dmtTc1p5VjIzZlNGN3paaXlUWTZUN0EwcCtnczUwNVhEOUdBYjlWYmR3R0cwK0tDVnl
        c1ZRZ1YySXdKZ2l5aHF3RUNvY3dCcmFuN251SytURU5EMmwyZjlZcng1b1JNRU56NzB3bHlIMzZPWkJtdDBrNTk4L1doMEhEWUxaZW8wZHlTV3JOd3dlWXZTeEU4L01kbTQzWEV1U3pialR6ZnNNMHZVUn
        GQlNyVUxFYURPMS9JUDJVcjdCc2dId1JJL3hmb3FJbUsxS2twVXEwQWxjVEFpM3YxdTl6Qy9xTGdQd0F5UUl2dzlVQ3NpcnJQQTBZMFlPaFFJREFRQUJNQTBHQ1NxR1NJYjNEUUVCQ3dVQUE0SUJBUURja
        9qZEd6L0FJQ2pqTTBaN21ZbGdQNXpic2FRNWRDMmNqZjRESnFta21zV3VmUnBDNHNic3VoODcwY0NCS2N1dmgrb0dpekJRSHJQbTRUaEl2ZklsS0w4WGpMQVhiRnVSUG9IQWcwOHNMWGR2UFRCVE52REYx
        WhvcU5zMmo2ZUZxL2ROdXF2ZUJIcjVENXRLblYyWEJHRUhFOVJFOVdUa1pRT2MwaEhtQ3dNbWNZb3JYRzhCa3l1OXFwNXhyMDZMQ0htMnJLcnI2ZENRVldBV0R0MzhiS2t5STRobTVSNTVCclR5UldSdzI
        RS9YaFEwVksva1FJYW9GZ0hvaWo0ekg5bmxlZnZMbmhaZDVPRzROL29OS2pBKy9LbkVqaTdPQXhKWVNaR1FmRjU0R1AwQTE4VnF1NVVGaFBKMUZFQXZ5YjR0QnZtTzM1NFFVUys5UTY2agotLS0tLUVORC
        DRVJUSUZJQ0FURS0tLS0t
    data_receiving: manually
    login_url: https://login.checkpointonline.com/621ac12d-4afb-479c-9c14-13e7b743cd07/saml2
    name: TestIdp
    received_identifier: https://sts.checkpoint.net/621ac12d-4afb-479c-9c14-13e7b743cd07/
    state: present
    usage: managing_administrator_access

- name: delete-identity-provider
  cp_mgmt_identity_provider:
    name: TestIdp
    state: absent
"""

RETURN = """
cp_mgmt_identity_provider:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        usage=dict(type='str', choices=['managing_administrator_access', 'gateway_policy_and_logs']),
        gateway=dict(type='str'),
        service=dict(type='str', choices=['none', 'identity awareness', 'mobile access', 'vpn']),
        data_receiving=dict(type='str', choices=['metadata_file', 'manually']),
        received_identifier=dict(type='str'),
        login_url=dict(type='str'),
        base64_metadata_file=dict(type='str'),
        base64_certificate=dict(type='str'),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        domains_to_process=dict(type='list', elements='str'),
        tags=dict(type='list', elements='str'),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'identity-provider'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()
